Package: clugenr
Title: Multidimensional Cluster Generation Using Support Lines
Version: 1.0.4
Authors@R: 
    person(given = "Nuno",
           family = "Fachada",
           role = c("aut", "cre", "cph"),
           email = "faken@fakenmc.com",
           comment = c(ORCID = "0000-0002-8487-5837"))
Description: An implementation of the clugen algorithm for generating
    multidimensional clusters with arbitrary distributions. Each cluster is
    supported by a line segment, the position, orientation and length of which
    guide where the respective points are placed. This package is described in
    Fachada & de Andrade (2023) <doi:10.1016/j.knosys.2023.110836>.
Depends: R (>= 3.6.0)
Imports: mathjaxr
Suggests: crul, devtools, ggplot2, knitr, lintr, patchwork, prettydoc,
        rgl, rmarkdown, roxygen2, testthat (>= 3.0.0)
RdMacros: mathjaxr
License: MIT + file LICENSE
URL: https://clugen.github.io/clugenr/,
        https://github.com/clugen/clugenr
BugReports: https://github.com/clugen/clugenr/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-02 14:39:23 UTC; hornik
Author: Nuno Fachada [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-8487-5837>)
Maintainer: Nuno Fachada <faken@fakenmc.com>
Repository: CRAN
Date/Publication: 2026-04-02 16:47:39
Built: R 4.6.0; ; 2026-04-26 14:47:48 UTC; unix

# Generated by roxygen2: do not edit by hand

export(angle_btw)
export(angle_deltas)
export(clucenters)
export(clugen)
export(clumerge)
export(clupoints_n)
export(clupoints_n_1)
export(clupoints_n_1_template)
export(clusizes)
export(fix_empty)
export(fix_num_points)
export(llengths)
export(points_on_line)
export(rand_ortho_vector)
export(rand_unit_vector)
export(rand_vector_at_angle)
importFrom(mathjaxr,preview_rd)

[![Checks](https://github.com/clugen/clugenr/actions/workflows/check.yml/badge.svg)](https://github.com/clugen/clugenr/actions/workflows/check.yml)
[![codecov](https://codecov.io/gh/clugen/clugenr/branch/main/graph/badge.svg?token=3MWG03OYS5)](https://app.codecov.io/gh/clugen/clugenr)
[![docs](https://img.shields.io/badge/docs-latest-blue.svg)](https://clugen.github.io/clugenr/)
[![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/clugenr)](https://CRAN.R-project.org/package=clugenr)
![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/clugenr)
[![MIT](https://img.shields.io/badge/license-MIT-yellowgreen.svg)](https://www.tldrlegal.com/license/mit-license)

# clugenr <img src="man/figures/logo.png" align="right" />

## Summary

**clugenr** is an [R] implementation of the *clugen* algorithm for generating
multidimensional clusters with arbitrary distributions. Each cluster is
supported by a line segment, the position, orientation and length of which guide
where the respective points are placed.

## How to install

A stable version of the package is available on [CRAN] and can be installed with
the following instruction:

```R
install.packages("clugenr")
```

Alternatively, install the development version from GitHub with the following
command (requires the [devtools] package):

```R
devtools::install_github("clugen/clugenr")
```

## Documentation

All methods and functions are fully documented and can be queried using the
built-in help system. After installation, to access the man pages, invoke the
`clugenr` help page as follows:

```R
help("clugenr")
```

The complete online documentation is available [here](https://clugen.github.io/clugenr/).

## Quick start

```R
library(clugenr)
x <- clugen(2, 5, 1000, c(1, -0.5), 0.5, c(4, 6), 6, 0.2, 0.5)
plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
```

![](man/figures/example2d.png)

```R
library(clugenr)
library(rgl) # For 3D plots
x <- clugen(3, 5, 2000, c(1, 0.5, -0.7), 0.5, c(15, 10, 20), 15, 3, 10)
plot3d(x$points, col = x$clusters, xlab = "x", ylab = "y", zlab = "z", aspect = T)
```

![](man/figures/example3d.png)

## See also

* [pyclugen](https://github.com/clugen/pyclugen/), a Python implementation of
  the *clugen* algorithm.
* [CluGen.jl](https://github.com/clugen/CluGen.jl/), an Julia implementation of
  the *clugen* algorithm.
* [MOCluGen](https://github.com/clugen/MOCluGen/), a MATLAB/Octave implementation
  of the *clugen* algorithm.

## Reference

If you use this software, please cite the following reference:

* Fachada, N. & de Andrade, D. (2023). Generating multidimensional clusters
  with support lines. *Knowledge-Based Systems*, 277, 110836.
  <https://doi.org/10.1016/j.knosys.2023.110836>
  ([arXiv preprint](https://doi.org/10.48550/arXiv.2301.10327))

## License

[MIT License](LICENSE)

[R]: https://www.r-project.org/
[devtools]: https://cran.r-project.org/package=devtools
[CRAN]: https://cran.r-project.org/

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

library(rgl)       # For plotting in 3D

# Function for plotting a series of related 3D examples
plot_examples_3d <- function(..., pmargin = 0.1, ncols = 3) {

  # Place examples in a list
  ets <- list(...)

  # Get maximum and minimum points in each dimension for all examples
  xmax <- max(sapply(ets, function(et) max(et$e$points[, 1])))
  xmin <- min(sapply(ets, function(et) min(et$e$points[, 1])))
  ymax <- max(sapply(ets, function(et) max(et$e$points[, 2])))
  ymin <- min(sapply(ets, function(et) min(et$e$points[, 2])))
  zmax <- max(sapply(ets, function(et) max(et$e$points[, 3])))
  zmin <- min(sapply(ets, function(et) min(et$e$points[, 3])))

  # Determine plots centers in each dimension
  xcenter <- (xmax + xmin) / 2
  ycenter <- (ymax + ymin) / 2
  zcenter <- (zmax + zmin) / 2

  # Determine plot span in all dimensions
  sidespan <- (1 + pmargin) * max(abs(xmax - xmin),
                                  abs(ymax - ymin),
                                  abs(zmax - zmin)) / 2

  # Determine final plots limits in all dimensions
  xmax <- xcenter + sidespan
  xmin <- xcenter - sidespan
  ymax <- ycenter + sidespan
  ymin <- ycenter - sidespan
  zmax <- zcenter + sidespan
  zmin <- zcenter - sidespan

  if (length(ets) %% ncols != 0) {
    stop("Number of examples must be a multiple of `ncols`")
  }

  # Determine number of rows for subplots in final plot
  nrows <- length(ets) %/% ncols

  # Combine plots as subplots with shared mouse and perspective
  mfrow3d(nrows, ncols, sharedMouse = TRUE)

  # Create plots
  lapply(
    ets,
    function(et) {
      e <- et$e
      t <- et$t
      plot3d(e$points, type = "s", size = 1.5,
             col = e$clusters, aspect = TRUE,
             xlab = "x", ylab = "y", zlab = "z", main = t,
             xlim = c(xmin, xmax), ylim = c(ymin, ymax), zlim = c(zmin, zmax))
    }
  )

  # Trigger display as rglwidget
  highlevel(integer())
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

library(ggplot2)   # For plotting in 2D
library(patchwork) # For combining 2D plots

# Function for plotting a series of related 2D examples
plot_examples_2d <- function(..., pmargin = 0.1) {

  # Place examples in a list
  ets <- list(...)

  # Get maximum and minimum points in each dimension for all examples
  xmax <- max(sapply(ets, function(et) max(et$e$points[, 1])))
  xmin <- min(sapply(ets, function(et) min(et$e$points[, 1])))
  ymax <- max(sapply(ets, function(et) max(et$e$points[, 2])))
  ymin <- min(sapply(ets, function(et) min(et$e$points[, 2])))

  # Determine plots centers in each dimension
  xcenter <- (xmax + xmin) / 2
  ycenter <- (ymax + ymin) / 2

  # Determine plots span in both dimensions
  sidespan <- (1 + pmargin) * max(abs(xmax - xmin), abs(ymax - ymin)) / 2

  # Determine final plots limits in both dimensions
  xmax <- xcenter + sidespan
  xmin <- xcenter - sidespan
  ymax <- ycenter + sidespan
  ymin <- ycenter - sidespan

  # Create plots
  plts <- lapply(
    ets,
    function(et) {
      e <- et$e
      t <- et$t
      ggplot(NULL, aes(x = e$points[, 1], y = e$points[, 2])) +
        geom_point(shape = 21, colour = "black", stroke = 0.1,
                   aes(fill = e$clusters)) +
        xlab(NULL) + ylab(NULL) + ggtitle(t) +
        theme(legend.position = "none",
              plot.title = element_text(size = rel(0.75))) +
        coord_fixed(xlim = c(xmin, xmax), ylim = c(ymin, ymax))
    })

  # Combine plots as subplots
  wrap_plots(plts)
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' Angle between two \mjseqn{n}-dimensional vectors.
#'
#' @description
#' \loadmathjax
#' Typically, the angle between two vectors `v1` and `v2` can be obtained with:
#'
#' ```R
#' acos((v1 %*% v2) / (norm(v1, "2") * norm(v2, "2")))
#' ```
#' However, this approach is numerically unstable. The version provided here is
#' numerically stable and based on the [Angle Between
#' Vectors](https://github.com/JeffreySarnoff/AngleBetweenVectors.jl)
#' Julia package by Jeffrey Sarnoff (MIT license), implementing an algorithm
#' provided by Prof. W. Kahan in
#' [these notes](https://people.eecs.berkeley.edu/~wkahan/MathH110/Cross.pdf)
#' (see page 15).
#'
#' @param v1 First vector.
#' @param v2 Second vector.
#' @return Angle between `v1` and `v2` in radians.
#'
#' @export
#'
#' @examples
#' angle_btw(c(1.0, 1.0, 1.0, 1.0), c(1.0, 0.0, 0.0, 0.0)) * 180 / pi
angle_btw <- function(v1, v2) {

  u1 <- v1 / norm(v1, "2")
  u2 <- v2 / norm(v2, "2")

  y <- u1 - u2
  x <- u1 + u2

  2 * atan(norm(y, "2") / norm(x, "2"))
}

#' Create points from their projections on a cluster-supporting line
#'
#' @description
#' \loadmathjax
#' Generate points from their \mjseqn{n}-dimensional projections on a
#' cluster-supporting line, placing each point on a hyperplane orthogonal to
#' that line and centered at the point's projection. The function specified in
#' `dist_fn` is used to perform the actual placement.
#'
#' @details
#' This function is used internally by [clupoints_n_1] and may be useful for
#' constructing user-defined final point placement strategies for the
#' `point_dist_fn` parameter of the main [clugen] function.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param projs Point projections on the cluster-supporting line
#' (\mjeqn{p \times n}{p x n} matrix).
#' @param lat_disp Dispersion of points from their projection.
#' @param clu_dir Direction of the cluster-supporting line (unit vector).
#' @param dist_fn Function to place points on a second line, orthogonal to the
#' first.
#' @return Generated points (\mjeqn{p \times n}{p x n} matrix).
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' ctr <- c(0, 0)
#' dir <- c(1, 0)
#' pdist <- c(-0.5, -0.2, 0.1, 0.3)
#' proj <- points_on_line(ctr, dir, pdist)
#' clupoints_n_1_template(proj, 0, dir, function(p, l) stats::runif(p))
clupoints_n_1_template <- function(projs, lat_disp, clu_dir, dist_fn) {

  # Number of dimensions
  num_dims <- length(clu_dir)

  # Number of points in this cluster
  clu_num_points <- nrow(projs)

  # Get distances from points to their projections on the line
  points_dist <- dist_fn(clu_num_points, lat_disp)

  # Get normalized vectors, orthogonal to the current line, for each point
  orth_vecs <- matrix(0, clu_num_points, num_dims)

  for (j in 1:clu_num_points) {
    orth_vecs[j, ] <- rand_ortho_vector(clu_dir)
  }

  # Set vector magnitudes
  orth_vecs <- abs(points_dist) * orth_vecs

  # Add perpendicular vectors to point projections on the line,
  # yielding final cluster points
  points <- projs + orth_vecs

  points
}

#' Certify that, given enough points, no clusters are left empty
#'
#' @description
#' \loadmathjax
#' Certifies that, given enough points, no clusters are left empty. This is done
#' by removing a point from the largest cluster and adding it to an empty
#' cluster while there are empty clusters. If the total number of points is
#' smaller than the number of clusters (or if the `allow_empty` parameter is set
#' to `TRUE`), this function does nothing.
#'
#' @details
#' This function is used internally by [clusizes] and might be useful for custom
#' cluster sizing implementations given as the `clusizes_fn` parameter of the
#' main [clugen] function.
#'
#' @param clu_num_points Number of points in each cluster (vector of size
#' \mjseqn{c}), where \mjseqn{c} is the number of clusters.
#' @param allow_empty Allow empty clusters?
#' @return Number of points in each cluster, after being fixed by this function
#' (vector of size \mjseqn{c}).
#'
#' @export
#'
#' @examples
#' clusters <- c(3, 4, 5, 0, 0)    # A vector with some empty elements
#' clusters <- fix_empty(clusters) # Apply this function
#' clusters                        # Check that there's no more empty elements
fix_empty <- function(clu_num_points, allow_empty = FALSE) {

  # If the allow_empty parameter is set to true, don't change the number of
  # points in each cluster; this is useful for quick `clusizes_fn` one-liners
  if (!allow_empty) {

    # Find empty clusters
    empty_clusts <- which(clu_num_points == 0)

    # If there are empty clusters and enough points for all clusters...
    if (length(empty_clusts) > 0 &&
        sum(clu_num_points) >= length(clu_num_points)) {

      # Go through the empty clusters...
      for (i0 in empty_clusts) {

        # ...get a point from the largest cluster and assign it to the
        # current empty cluster
        imax <- which.max(clu_num_points)
        clu_num_points[imax] <- clu_num_points[imax] - 1
        clu_num_points[i0] <- 1
      }
    }
  }

  clu_num_points
}

#' Certify that array values add up to a specific total
#'
#' @description
#' \loadmathjax
#' Certifies that the values in the `clu_num_points` array, i.e. the number of
#' points in each cluster, add up to `num_points`. If this is not the case, the
#' `clu_num_points` array is modified in-place, incrementing the value
#' corresponding to the smallest cluster while
#' `sum(clu_num_points) < num_points`, or decrementing the value corresponding
#' to the largest cluster while `sum(clu_num_points) > num_points`.
#'
#' @details
#' This function is used internally by [clusizes] and might be useful for
#' custom cluster sizing implementations given as the `clusizes_fn` parameter of
#' the main [clugen] function.
#'
#' @param clu_num_points Number of points in each cluster (vector of size
#' \mjseqn{c}), where \mjseqn{c} is the number of clusters.
#' @param num_points The expected total number of points.
#' @return Number of points in each cluster, after being fixed by this function.
#'
#' @export
#'
#' @examples
#' clusters <- c(1, 6, 3)                   # 10 total points
#' clusters <- fix_num_points(clusters, 12) # But we want 12 total points
#' clusters                                 # Check that we now have 12 points
fix_num_points <- function(clu_num_points, num_points) {

  while (sum(clu_num_points) < num_points) {
    imin <- which.min(clu_num_points)
    clu_num_points[imin] <- clu_num_points[imin] + 1
  }
  while (sum(clu_num_points) > num_points) {
    imax <- which.max(clu_num_points)
    clu_num_points[imax] <- clu_num_points[imax] - 1
  }

  clu_num_points
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# ################################## #
# Test clugen() mandatory parameters #
# ################################## #

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims,
                     nclu = num_clusters,
                     tpts = num_points,
                     astd = angles_stds,
                     len = llengths_mus,
                     len_std = llengths_sigmas,
                     lat_std = lat_stds)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]
  nclu <- targs[i, "nclu"]
  tpts <- targs[i, "tpts"]
  astd <- targs[i, "astd"]
  len <- targs[i, "len"]
  len_std <- targs[i, "len_std"]
  lat_std <- targs[i, "lat_std"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (line directions + clu seps)
  tsargs <- expand.grid(direc = append(asplit(get_vecs(ndirs, nd), 1),
                                       list(matrix(rnorm(nclu * nd),
                                                   ncol = nd))),
                        clusep = asplit(get_clu_seps(nd), 1))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current line direction + line center
    direc <- tsargs[j, "direc"][[1]]
    clusep <- tsargs[j, "clusep"][[1]]

    # Determine test name for current parameter set
    test_desc <- paste0("clugen mandatory params: ",
                        "seed=", seed, ", nd=", nd, ", nclu=", nclu,
                        ", tpts=", tpts, ", astd=", astd, ", len=", len,
                        ", len_std=", len_std, ", lat_std=", lat_std,
                        ", direc=[", paste(direc, collapse = ", "),
                        "], clusep=[", paste(clusep, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # By default, allow_empty is false, so clugen() must be given more points
      # than clusters...
      if (tpts >= nclu) {
        # ...in which case it runs without problem
        expect_warning(r <- clugen(nd, nclu, tpts, direc, astd,
                                   clusep, len, len_std, lat_std),
                       regexp = NA)

        # Check dimensions of result variables
        expect_equal(dim(r$points), c(tpts, nd))
        expect_equal(length(r$clusters), tpts)
        expect_equal(dim(r$projections), c(tpts, nd))
        expect_equal(length(r$sizes), nclu)
        expect_equal(dim(r$centers), c(nclu, nd))
        expect_equal(dim(r$directions), c(nclu, nd))
        expect_equal(length(r$angles), nclu)
        expect_equal(length(r$lengths), nclu)

        # Check point cluster indexes
        expect_equal(unique(as.numeric(r$clusters)), 1:nclu)

        # Check total points
        expect_equal(sum(r$sizes), tpts)

        # Check that cluster directions have the correct angles with the main
        # direction
        if (nd > 1) {
          # In case direction is just a vector, repeat it num_cluster times
          # into a matrix...
          if (is.vector(direc) ||
              (is.array(direc) && length(dim(direc)) == 1)) {
            direc <- matrix(direc,
                            nrow = nclu,
                            ncol = length(direc),
                            byrow = TRUE)
          }
          # ...so we can check each cluster direction separately
          for (i in 1:nclu) {
            expect_equal(angle_btw(direc[i, ], r$directions[i, ]),
                         abs(r$angles[i]))
          }
        }

      } else {
        # ...otherwise an error will be thrown
        expect_error(r <- clugen(nd, nclu, tpts, direc, astd,
                                 clusep, len, len_std, lat_std),
                     paste("A total of", tpts, "points is not enough for", nclu,
                           "non-empty clusters"))
      }
    })
  }
}

# ################################# #
# Test clugen() optional parameters #
# ################################# #

# Valid arguments
nclu <- 7
tpts <- 500
astd <- pi / 256
len_mu <- 9
len_std <- 1.2
lat_std <- 2

# Alternative functions used for testing
prjdist_equi <- function(l, n) seq(from = -l / 2, to = l / 2, length.out = n)
ptdist_projs_plus_1 <- function(projs, ls, l, cd, cc) projs + 1
csz_equi <- function(nclu, tpts, ae) {
  cs <- vector(mode = "integer", length = nclu)
  for (i in 1:tpts) {
    cs[i %% nclu + 1] <- cs[i %% nclu + 1] + 1
  }
  cs
}
cctr_on_a_line <- function(nclu, cs, co) {
  1:nclu * matrix(0, nrow = nclu, ncol = length(cs))
}
llen_unif_10_20 <- function(nclu, l, lsd) stats::runif(nclu, min = 10, max = 20)
lang_same <- function(nclu, astd) vector(mode = "double", length = nclu)

# Create parameter combinations to test
if (testthat:::on_cran() || is_test_mode("cran")) {
  # Light tests, for CRAN
  targs <- expand.grid(seed = seeds, nd = 2, ae = TRUE,
                       prjdist_fn = list("norm", "unif"),
                       ptdist_fn = list("n-1", "n"),
                       csz_fn = list(clusizes),
                       cctr_fn = list(clucenters),
                       llen_fn = list(llengths),
                       lang_fn = list(angle_deltas))
} else {
  # Other testing modes will be more thorough
  targs <- expand.grid(seed = seeds, nd = c(2, 7), ae = allow_empties,
                       prjdist_fn = list("norm", "unif", prjdist_equi),
                       ptdist_fn = list("n-1", "n", ptdist_projs_plus_1),
                       csz_fn = list(clusizes, csz_equi),
                       cctr_fn = list(clucenters, cctr_on_a_line),
                       llen_fn = list(llengths, llen_unif_10_20),
                       lang_fn = list(angle_deltas, lang_same))
}

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]
  ae <- targs[i, "ae"]
  prjdist_fn <- targs[i, "prjdist_fn"][[1]]
  ptdist_fn <- targs[i, "ptdist_fn"][[1]]
  csz_fn <- targs[i, "csz_fn"][[1]]
  cctr_fn <- targs[i, "cctr_fn"][[1]]
  llen_fn <- targs[i, "llen_fn"][[1]]
  lang_fn <- targs[i, "lang_fn"][[1]]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (line directions + clu seps)
  tsargs <- expand.grid(direc = asplit(get_vecs(ndirs, nd), 1),
                        clusep = asplit(get_clu_seps(nd), 1))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current line direction + line center
    direc <- tsargs[j, "direc"][[1]]
    clusep <- tsargs[j, "clusep"][[1]]

    # Determine test name for current parameter set
    test_desc <- paste0("clugen optional params: ",
                        "seed = ", seed, ", nd=", nd, ", nclu=", nclu,
                        ", tpts=", tpts, "ae=", ae,
                        ", ptdist_fn='",
                        format(prjdist_fn)[length(format(prjdist_fn))],
                        "', ptoff_fn='",
                        format(ptdist_fn)[length(format(ptdist_fn))],
                        "', czn_fn='",
                        format(csz_fn)[length(format(csz_fn))],
                        "'",
                        "', cctr_fn='",
                        format(cctr_fn)[length(format(cctr_fn))],
                        "'",
                        "', llen_fn='",
                        format(llen_fn)[length(format(llen_fn))],
                        "'",
                        "', lang_fn='",
                        format(lang_fn)[length(format(lang_fn))],
                        "'")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # ...in which case it runs without problem
      expect_warning(r <- clugen(nd, nclu, tpts, direc, astd,
                                 clusep, len_mu, len_std, lat_std,
                                 allow_empty = ae,
                                 proj_dist_fn = prjdist_fn,
                                 point_dist_fn = ptdist_fn,
                                 clusizes_fn = csz_fn,
                                 clucenters_fn = cctr_fn,
                                 llengths_fn = llen_fn,
                                 angle_deltas_fn = lang_fn,
                                 seed = seed),
                     regexp = NA)

      # Check dimensions of result variables
      expect_equal(dim(r$points), c(tpts, nd))
      expect_equal(length(r$clusters), tpts)
      expect_equal(dim(r$projections), c(tpts, nd))
      expect_equal(length(r$sizes), nclu)
      expect_equal(dim(r$centers), c(nclu, nd))
      expect_equal(dim(r$directions), c(nclu, nd))
      expect_equal(length(r$angles), nclu)
      expect_equal(length(r$lengths), nclu)

      # Check point cluster indexes
      if (!ae) {
        expect_equal(unique(as.numeric(r$clusters)), 1:nclu)
      } else {
        expect_true(all(as.numeric(r$clusters) <= nclu))
      }

      # Check total points
      expect_equal(sum(r$sizes), tpts)
      # This might not be the case if the specified clusize_fn does not obey
      # the total number of points

      # Check that cluster directions have the correct angles with the main
      # direction
      if (nd > 1) {
        for (i in 1:nclu) {
          expect_equal(angle_btw(direc, r$directions[i, ]),
                       abs(r$angles[i]))
        }
      }
    })
  }
}

# ######################################## #
# Test clugen() optional direct parameters #
# ######################################## #

# Valid arguments
astd <- pi / 333
len_mu <- 6
len_std <- 1.1
lat_std <- 1.6

# Create parameter combinations to test
if (testthat:::on_cran() || is_test_mode("cran")) {
  # Light tests, for CRAN
  targs <- expand.grid(seed = seeds, nd = 2, nclu = 6)
} else {
  # Other testing modes will be more thorough
  targs <- expand.grid(seed = seeds, nd = c(1, 5), nclu = c(1, 6))
}

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]
  nclu <- targs[i, "nclu"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters
  direc <- rnorm(nd)
  clusep <- 100 * rnorm(nd)
  csz_direct <- sample(1:100, nclu, replace = TRUE)
  cctr_direct <- matrix(100 * rnorm(nd * nclu), nrow = nclu)
  llen_direct <- runif(nclu, min = 0, max = 100)
  lang_direct <- runif(nclu, min = -pi / 2, max = pi / 2)
  tpts <- sum(csz_direct)

  # Determine test name for current parameter set
  test_desc <- paste0("clugen optional params (direct): ",
                      "seed = ", seed, ", nd=", nd, ", nclu=", nclu,
                      ", direc=[", paste(direc, collapse = ", "),
                      ", clusep=[", paste(clusep, collapse = ", "),
                      ", csz_direct=[", paste(csz_direct, collapse = ", "),
                      ", cctr_direct=[", paste(cctr_direct, collapse = ", "),
                      ", llen_direct=[", paste(llen_direct, collapse = ", "),
                      ", lang_direct=[", paste(lang_direct, collapse = ", "))

  # Perform tests for current parameter set
  test_that(test_desc, {

    # ...in which case it runs without problem
    expect_warning(r <- clugen(nd, nclu, tpts, direc, astd,
                               clusep, len_mu, len_std, lat_std,
                               allow_empty = ae,
                               clusizes_fn = csz_direct,
                               clucenters_fn = cctr_direct,
                               llengths_fn = llen_direct,
                               angle_deltas_fn = lang_direct,
                               seed = seed),
                   regexp = NA)

    # Check dimensions of result variables
    expect_equal(dim(r$points), c(tpts, nd))
    expect_equal(length(r$clusters), tpts)
    expect_equal(dim(r$projections), c(tpts, nd))
    expect_equal(length(r$sizes), nclu)
    expect_equal(dim(r$centers), c(nclu, nd))
    expect_equal(dim(r$directions), c(nclu, nd))
    expect_equal(length(r$angles), nclu)
    expect_equal(length(r$lengths), nclu)

    # Check point cluster indexes
    if (!ae) {
      expect_equal(unique(as.numeric(r$clusters)), 1:nclu)
    } else {
      expect_true(all(as.numeric(r$clusters) <= nclu))
    }

    # Check total points
    expect_equal(sum(r$sizes), tpts)
    # This might not be the case if the specified clusize_fn does not obey
    # the total number of points

    # Check that cluster directions have the correct angles with the main
    # direction
    if (nd > 1) {
      for (i in 1:nclu) {
        expect_equal(angle_btw(direc, r$directions[i, ]),
                     abs(r$angles[i]))
      }
    }
  })
}

# ##################### #
# Reproducibility tests #
# ##################### #

# Valid parameters
nd <- 2
nclu <- 4
tpts <- 300
direc <- c(1, 1)
astd <- pi / 64
clusep <- c(7, 6.5)
len_mu <- 4.1
len_std <- 0.5
lat_std <- 0.2

for (seed in seeds) {

  test_that(paste0("clugen reproducibility: seed=", seed), {

    # Get results for run 1 with current seed
    expect_warning(r1 <- clugen(nd, nclu, tpts, direc, astd, clusep,
                                len_mu, len_std, lat_std,
                                seed = seed),
                   regexp = NA)

    # Get results for run 2 with current seed
    expect_warning(r2 <- clugen(nd, nclu, tpts, direc, astd, clusep,
                                len_mu, len_std, lat_std,
                                seed = seed),
                   regexp = NA)

    # Check that results are exactly the same
    expect_equal(r1, r2)
  })
}

# ######################## #
# Test clugen() exceptions #
# ######################## #

for (seed in seeds) {

  # Valid parameters
  nd <- 3
  nclu <- 5
  tpts <- 1000
  direc <- c(1, 0, 0)
  astd <- pi / 64
  clusep <- c(10, 10, 5)
  len_mu <- 5
  len_std <- 0.5
  lat_std <- 0.3
  ae <- TRUE
  clu_off <- c(-1.5, 0, 2)
  prj_dist <- "unif"
  pt_dist <- "n-1"
  csizes_fn <- clusizes
  ccenters_fn <- clucenters
  llengths_fn <- llengths
  langles_fn <- angle_deltas

  test_that(paste0("clugen exceptions: seed=", seed), {

    # Test passes with valid arguments
    expect_warning(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                               len_mu, len_std, lat_std,
                               allow_empty = ae,
                               cluster_offset = clu_off,
                               proj_dist_fn = prj_dist,
                               point_dist_fn = pt_dist,
                               clusizes_fn = csizes_fn,
                               clucenters_fn = ccenters_fn,
                               llengths_fn = llengths_fn,
                               angle_deltas_fn = langles_fn,
                               seed = seed),
                   regexp = NA)

    # Test passes with zero points since allow_empty is set to true
    expect_warning(r <- clugen(nd, nclu, 0, direc, astd, clusep,
                               len_mu, len_std, lat_std,
                               allow_empty = ae,
                               cluster_offset = clu_off,
                               proj_dist_fn = prj_dist,
                               point_dist_fn = pt_dist,
                               clusizes_fn = csizes_fn,
                               clucenters_fn = ccenters_fn,
                               llengths_fn = llengths_fn,
                               angle_deltas_fn = langles_fn,
                               seed = seed),
                   regexp = NA)

    # Invalid number of dimensions
    expect_error(r <- clugen(0, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = "Number of dimensions, `num_dims`, must be > 0",
                 fixed = TRUE)

    # Invalid number of clusters
    expect_error(r <- clugen(nd, 0, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = "Number of clusters, `num_clust`, must be > 0",
                 fixed = TRUE)

    # Direction needs to have magnitude > 0
    expect_error(r <- clugen(nd, nclu, tpts, c(0, 0, 0), astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = "`direction` must have magnitude > 0",
                 fixed = TRUE)

    # Direction needs to have nd dims
    bad_dir <- c(1, 1)
    expect_error(r <- clugen(nd, nclu, tpts, bad_dir, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0(
                   "Length of directions in `direction` must be equal to ",
                   "`num_dims` (", length(bad_dir), " != ", nd, ")"),
                 fixed = TRUE)

    # Specific direction for each cluster requires one direction per cluster
    expect_error(r <- clugen(nd, nclu, tpts,
                             # but we're passing one extra direction
                             matrix(rnorm(nd * (nclu + 1)), ncol = nd),
                             astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("Number of rows in `direction` must be the ",
                                 "same as the number of clusters (",
                                 nclu + 1, " != ", nclu, ")"),
                 fixed = TRUE)

    # Direction needs to be a 1D array (vector) or 2D array (matrix)
    expect_error(r <- clugen(nd, nclu, tpts,
                               # but we're passing a 3D array
                               array(rnorm(nd * nclu * 2), c(nd, nclu, 2)),
                               astd, clusep, len_mu, len_std, lat_std,
                               allow_empty = ae,
                               cluster_offset = clu_off,
                               proj_dist_fn = prj_dist,
                               point_dist_fn = pt_dist,
                               clusizes_fn = csizes_fn,
                               clucenters_fn = ccenters_fn,
                               llengths_fn = llengths_fn,
                               angle_deltas_fn = langles_fn,
                               seed = seed),
                   regexp = paste0("`direction` must be a vector (1D array) ",
                                   "or a matrix (2D array)"),
                   fixed = TRUE)

    # cluster_sep needs to have nd dims
    bad_clusep <- c(10, 10)
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd,
                             bad_clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0(
                   "Length of `cluster_sep` must be equal to `num_dims` (",
                   length(bad_clusep), " != ", nd, ")"),
                 fixed = TRUE)

    # cluster_offset needs to have nd dims
    bad_cluoff <- c(0, 1)
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = bad_cluoff,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("Length of `cluster_offset` must be equal to ",
                                 "`num_dims` (",
                                 length(bad_cluoff), " != ", nd, ")"),
                 fixed = TRUE)

    # Unknown proj_dist_fn given as string
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = "invalid",
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0(
                   "`proj_dist_fn` has to be either \"norm\", \"unif\" or",
                   " user-defined function"),
                 fixed = TRUE)

    # Invalid proj_dist_fn given as function
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = csizes_fn,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 "argument")

    # Unknown point_dist_fn given as string
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = "invalid",
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("point_dist_fn has to be either \"n-1\",",
                                 " \"n\" or a user-defined function"),
                 fixed = TRUE)

    # Invalid point_dist_fn given as function
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = llengths_fn,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = "argument")

    # Invalid direct clusizes
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = sample(1:(tpts * nclu), nclu + 1),
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("`clusizes_fn` has to be either a function or",
                                 " a `num_clusters`-sized vector"),
                 fixed = TRUE)

    # Invalid direct clucenters
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = matrix(rnorm(nd * (nclu + 1)),
                                                    nrow = nclu + 1),
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("clucenters_fn has to be either a function or",
                                 " a matrix of size `num_clusters` x `num_dims`"),
                 fixed = TRUE)

    # Invalid direct llengths
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = runif(nclu + 1),
                             angle_deltas_fn = langles_fn,
                             seed = seed),
                 regexp = paste0("`llengths_fn` has to be either a function or",
                                 " a `num_clusters`-sized vector"),
                 fixed = TRUE)

    # Invalid direct langles
    expect_error(r <- clugen(nd, nclu, tpts, direc, astd, clusep,
                             len_mu, len_std, lat_std,
                             allow_empty = ae,
                             cluster_offset = clu_off,
                             proj_dist_fn = prj_dist,
                             point_dist_fn = pt_dist,
                             clusizes_fn = csizes_fn,
                             clucenters_fn = ccenters_fn,
                             llengths_fn = llengths_fn,
                             angle_deltas_fn = runif(nclu + 1),
                             seed = seed),
                 regexp = paste0("`angle_deltas_fn` has to be either a function",
                                 " or a `num_clusters`-sized vector"),
                 fixed = TRUE)

  })
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

test_that("angle_btw works as expected", {

  # Commonly used function for determining the angle between two vectors
  common_angle_btw <- function(v1, v2) {
    acos((v1 %*% v2) / (norm(v1, "2") * norm(v2, "2")))[1]
  }

  # 2D
  u <- c(1.5, 0)
  v <- c(0.1, -0.4)
  expect_equal(angle_btw(u, v), common_angle_btw(u, v))

  # 3D
  u <- c(-1.5, 10, 0)
  v <- c(0.99, 4.4, -1.1)
  expect_equal(angle_btw(u, v), common_angle_btw(u, v))

  # 8D
  u <- c(1.5, 0, 0, 0, 0, 0, 0, -0.5)
  v <- c(7.5, -0.4, 0, 0, 0, -16.4, 0.1, -0.01)
  expect_equal(angle_btw(u, v), common_angle_btw(u, v))
})

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

test_that("fix_num_points works as expected", {

  # No change
  clusts <- c(10, 100, 42, 0, 12)
  expect_warning(clusts_fixed <- fix_num_points(clusts, sum(clusts)),
                 regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # Fix due to too many points
  clusts <- c(55, 12)
  num_pts <- sum(clusts) - 14
  expect_warning(clusts_fixed <- fix_num_points(clusts, num_pts), regexp = NA)
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_equal(sum(clusts_fixed), num_pts)

  # Fix due to too few points
  clusts <- c(0, 1, 0, 0)
  num_pts <- 15
  expect_warning(clusts_fixed <- fix_num_points(clusts, num_pts), regexp = NA)
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_equal(sum(clusts_fixed), num_pts)

  # 1D - No change
  clusts <- 10
  expect_warning(clusts_fixed <- fix_num_points(clusts, sum(clusts)),
                 regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # 1D - Fix due to too many points
  clusts <- 241
  num_pts <- sum(clusts) - 20
  expect_warning(clusts_fixed <- fix_num_points(clusts, num_pts), regexp = NA)
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_equal(sum(clusts_fixed), num_pts)

  # 1D - Fix due to too few points
  clusts <- 0
  num_pts <- 8
  expect_warning(clusts_fixed <- fix_num_points(clusts, num_pts), regexp = NA)
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_equal(sum(clusts_fixed), num_pts)

})

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

test_that("fix_empty works as expected", {

  #  No empty clusters
  clusts <- c(11, 21, 10)
  expect_warning(clusts_fixed <- fix_empty(clusts, allow_empty = FALSE),
                 regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # Empty clusters, no fix
  clusts <- c(0, 11, 21, 10, 0, 0)
  expect_warning(clusts_fixed <- fix_empty(clusts, allow_empty = TRUE),
                 regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # Empty clusters, fix
  clusts <- c(5, 0, 21, 10, 0, 0, 101)
  expect_warning(clusts_fixed <- fix_empty(clusts), regexp = NA)
  expect_equal(sum(clusts), sum(clusts_fixed))
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_false(any(clusts_fixed == 0))

  # Empty clusters, fix, several equal maximums
  clusts <- c(101, 5, 0, 21, 101, 10, 0, 0, 101, 100, 99, 0, 0, 0, 100)
  expect_warning(clusts_fixed <- fix_empty(clusts), regexp = NA)
  expect_equal(sum(clusts), sum(clusts_fixed))
  expect_false(isTRUE(all.equal(clusts, clusts_fixed)))
  expect_false(any(clusts_fixed == 0))

  # Empty clusters, no fix (flag)
  clusts <- c(0, 10)
  expect_warning(clusts_fixed <- fix_empty(clusts, allow_empty = TRUE),
                 regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # Empty clusters, no fix (not enough points)
  clusts <- c(0, 1, 1, 0, 0, 2, 0, 0)
  expect_warning(clusts_fixed <- fix_empty(clusts), regexp = NA)
  expect_equal(clusts, clusts_fixed)

  # Works with 1D
  clusts <- 100
  expect_warning(clusts_fixed <- fix_empty(clusts), regexp = NA)
  expect_equal(clusts, clusts_fixed)

})

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' Generate multidimensional clusters
#'
#' @description
#' \loadmathjax
#' This is the main function of **clugenr**, and possibly the only function most
#' users will need.
#'
#' @details
#' If a custom function was given in the `clusizes_fn` parameter, it is
#' possible that `num_points` may have a different value than what was
#' specified in the `num_points` parameter.
#'
#' The terms "average" and "dispersion" refer to measures of central
#' tendency and statistical dispersion, respectively. Their exact meaning
#' depends on the optional arguments.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_dims Number of dimensions.
#' @param num_clusters Number of clusters to generate.
#' @param num_points Total number of points to generate.
#' @param direction Average direction of the cluster-supporting lines. Can be
#' a vector of length `num_dims` (same direction for all clusters) or a
#' matrix of size `num_clusters` x `num_dims` (one direction per cluster).
#' @param angle_disp Angle dispersion of cluster-supporting lines (radians).
#' @param cluster_sep Average cluster separation in each dimension (vector of
#' length `num_dims`).
#' @param llength Average length of cluster-supporting lines.
#' @param llength_disp Length dispersion of cluster-supporting lines.
#' @param lateral_disp Cluster lateral dispersion, i.e., dispersion of points
#' from their projection on the cluster-supporting line.
#' @param allow_empty Allow empty clusters? `FALSE` by default.
#' @param cluster_offset Offset to add to all cluster centers (vector of length
#' `num_dims`). By default there will be no offset.
#' @param proj_dist_fn Distribution of point projections along
#' cluster-supporting lines, with three possible values:
#' - `"norm"` (default): Distribute point projections along lines using a normal
#'   distribution (\mjeqn{\mu=}{μ=} _line_center_,
#'   \mjeqn{\sigma=}{σ=} `llength/6` ).
#' - `"unif"`: Distribute points uniformly along the line.
#' - User-defined function, which accepts two parameters, line length (`double`)
#'   and number of points (`integer`), and returns a vector containing the
#'   distance of each point projection to the center of the line. For example,
#'   the `"norm"` option roughly corresponds to
#'   `function(l, n) stats::rnorm(n, sd = l / 6)`.
#' @param point_dist_fn Controls how the final points are created from their
#' projections on the cluster-supporting lines, with three possible values:
#' - `"n-1"` (default): Final points are placed on a hyperplane orthogonal to
#'   the cluster-supporting line, centered at each point's projection, using the
#'   normal distribution (\mjeqn{\mu=0}{μ=0},
#'   \mjeqn{\sigma=}{σ=} `lateral_disp` ). This is done by the [clupoints_n_1]
#'   function.
#' - `"n"`: Final points are placed around their projection on the
#'   cluster-supporting line using the normal distribution (\mjeqn{\mu=0}{μ=0},
#'   \mjeqn{\sigma=}{σ=} `lateral_disp` ). This is done by the [clupoints_n]
#'   function.
#' - User-defined function: The user can specify a custom point placement
#'   strategy by passing a function with the same signature as [clupoints_n_1]
#'   and [clupoints_n].
#' @param clusizes_fn Distribution of cluster sizes. By default, cluster sizes
#' are determined by the [clusizes] function, which uses the normal distribution
#' (\mjeqn{\mu=}{μ=} `num_points`/`num_clusters`, \mjeqn{\sigma=\mu/3}{σ=μ/3}),
#' and assures that the final cluster sizes add up to `num_points`. This
#' parameter allows the user to specify a custom function for this purpose,
#' which must follow [clusizes] signature. Note that custom functions are not
#' required to strictly obey the `num_points` parameter. Alternatively, the user
#' can specify a vector of cluster sizes directly.
#' @param clucenters_fn Distribution of cluster centers. By default, cluster
#' centers are determined by the [clucenters] function, which uses the uniform
#' distribution, and takes into account the `num_clusters` and `cluster_sep`
#' parameters for generating well-distributed cluster centers. This parameter
#' allows the user to specify a custom function for this purpose, which must
#' follow [clucenters] signature. Alternatively, the user can specify a matrix
#' of size `num_clusters` x `num_dims` with the exact cluster centers.
#' @param llengths_fn Distribution of line lengths. By default, the lengths of
#' cluster-supporting lines are determined by the [llengths] function, which
#' uses the folded normal distribution (\mjeqn{\mu=}{μ=} `llength`,
#' \mjeqn{\sigma=}{σ=} `llength_disp` ). This parameter allows the user to
#' specify a custom function for this purpose, which must follow [llengths]
#' signature. Alternatively, the user can specify a vector of line lengths
#' directly.
#' @param angle_deltas_fn Distribution of line angle differences with respect to
#' `direction`. By default, the angles between the main `direction` of each
#' cluster and the final directions of their cluster-supporting lines are
#' determined by the [angle_deltas] function, which uses the wrapped normal
#' distribution (\mjeqn{\mu=0}{μ=0}, \mjeqn{\sigma=}{σ=} `angle_disp` ) with
#' support in the interval \mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}. This
#' parameter allows the user to specify a custom function for this purpose,
#' which must follow [angle_deltas] signature. Alternatively, the user can
#' specify a vector of angle deltas directly.
#' @param seed An integer used to initialize the PRNG, allowing for reproducible
#' results. If specified, `seed` is simply passed to [set.seed].
#' @return A named list with the following elements:
#' - `points`: A `num_points` x `num_dims` matrix with the generated points for
#'   all clusters.
#' - `clusters`: A `num_points` factor vector indicating which cluster
#'    each point in `points` belongs to.
#' - `projections`: A `num_points` x `num_dims` matrix with the point
#'   projections on the cluster-supporting lines.
#' - `sizes`: A `num_clusters` x 1 vector with the number of points in
#'   each cluster.
#' - `centers`: A `num_clusters` x `num_dims` matrix with the
#'   coordinates of the cluster centers.
#' - `directions`: A `num_clusters` x `num_dims` matrix with the final
#'   direction of each cluster-supporting line.
#' - `angles`: A `num_clusters` x 1 vector with the angles between the
#'   cluster-supporting lines and the main direction.
#' - `lengths`: A `num_clusters` x 1 vector with the lengths of the
#'   cluster-supporting lines.
#'
#' @export
#' @examples
#' # 2D example
#' x <- clugen(2, 5, 1000, c(1, 3), 0.5, c(10, 10), 8, 1.5, 2)
#' graphics::plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
#' # 3D example
#' x <- clugen(3, 5, 1000, c(2, 3, 4), 0.5, c(15, 13, 14), 7, 1, 2)
clugen <- function(num_dims, num_clusters, num_points, direction, angle_disp,
  cluster_sep, llength, llength_disp, lateral_disp,
  allow_empty = FALSE, cluster_offset = NA, proj_dist_fn = "norm",
  point_dist_fn = "n-1", clusizes_fn = clusizes, clucenters_fn = clucenters,
  llengths_fn = llengths, angle_deltas_fn = angle_deltas, seed = NA) {

  # ############### #
  # Validate inputs #
  # ############### #

  # Check that number of dimensions is > 0
  if (num_dims < 1) {
    stop("Number of dimensions, `num_dims`, must be > 0")
  }

  # Check that number of clusters is > 0
  if (num_clusters < 1) {
    stop("Number of clusters, `num_clust`, must be > 0")
  }

  # Is direction a vector or a matrix?
  if (is.vector(direction) ||
      (is.array(direction) && length(dim(direction)) == 1)) {
    # If a main direction vector was given, transpose it, so we can treat it
    # like a matrix later
    direction <- matrix(direction, ncol = length(direction))
  } else if (is.matrix(direction)) {
    # If a matrix was given (i.e. a main direction is given for each cluster),
    # check if the number of directions is the same as the number of clusters
    dir_size_1 <- dim(direction)[1]
    if (dir_size_1 != num_clusters) {
      stop("Number of rows in `direction` must be the same as the number of ",
           "clusters (", dir_size_1, " != ", num_clusters, ")")
    }
  } else {
    # The `direction` array must be a vector or a matrix, so if we get here
    # it means we have invalid arguments
    stop("`direction` must be a vector (1D array) or a matrix (2D array)")
  }

  # Check that `direction` has num_dims dimensions
  dir_size_2 <- dim(direction)[2]
  if (dir_size_2 != num_dims) {
    stop("Length of directions in `direction` must be equal to `num_dims` (",
         dir_size_2, " != ", num_dims, ")")
  }

  # Check that all directions in `direction` have magnitude > 0
  if (isTRUE(all.equal(min(apply(direction, 1, norm, type = "2")), 0))) {
    stop("`direction` must have magnitude > 0")
  }

  # If allow_empty is false, make sure there are enough points to distribute
  # by the clusters
  if (!allow_empty && num_points < num_clusters) {
    stop("A total of ", num_points, " points is not enough for ",
         num_clusters, " non-empty clusters")
  }

  # Check that cluster_sep has num_dims dimensions
  if (length(cluster_sep) != num_dims) {
    stop("Length of `cluster_sep` must be equal to `num_dims` (",
      length(cluster_sep), " != ", num_dims, ")")
  }

  # If given, cluster_offset must have the correct number of dimensions,
  # if not given then it will be a num_dims x 1 vector of zeros
  if (length(cluster_offset) == 1 && is.na(cluster_offset)) {
    cluster_offset <- vector(mode = "integer", length = num_dims)
  } else if (length(cluster_offset) != num_dims) {
    stop("Length of `cluster_offset` must be equal to `num_dims` (",
      length(cluster_offset), " != ", num_dims, ")")
  }

  # Check that proj_dist_fn specifies a valid way for projecting points along
  # cluster-supporting lines i.e., either "norm" (default), "unif" or a
  # user-defined function
  if (is.function(proj_dist_fn)) {
    # Use user-defined distribution; assume function accepts length of line
    # and number of points, and returns a number of points x 1 vector
    pointproj_fn <- proj_dist_fn
  } else if (is.character(proj_dist_fn) && proj_dist_fn == "unif") {
    # Point projections will be uniformly placed along cluster-supporting lines
    pointproj_fn <- function(l, n) stats::runif(n, min = -l / 2, max = l / 2)
  } else if (is.character(proj_dist_fn) && proj_dist_fn == "norm") {
    # Use normal distribution for placing point projections along
    # cluster-supporting lines, mean equal to line center, standard deviation
    # equal to 1/6 of line length such that the line length contains ≈99.73% of
    # the points
    pointproj_fn <- function(l, n) stats::rnorm(n, sd = (1.0 / 6.0) * l)
  } else {
    stop("`proj_dist_fn` has to be either \"norm\", \"unif\" or ",
         "user-defined function")
  }

  # Check that point_dist_fn specifies a valid way for generating points given
  # their projections along cluster-supporting lines, i.e., either "n-1"
  # (default), "n" or a user-defined function
  if (num_dims == 1) {
    # If 1D was specified, point projections are the points themselves
    pt_from_proj_fn <- function(projs, lat_disp, len, clu_dir, clu_ctr) projs
  } else if (is.function(point_dist_fn)) {
    # Use user-defined distribution; assume function accepts point projections
    # on the line, lateral disp., cluster direction and cluster center, and
    # returns a num_points x num_dims matrix containing the final points
    # for the current cluster
    pt_from_proj_fn <- point_dist_fn
  } else if (is.character(point_dist_fn) && point_dist_fn == "n-1") {
    # Points will be placed on a hyperplane orthogonal to the cluster-supporting
    # line using a normal distribution centered at their intersection
    pt_from_proj_fn <- clupoints_n_1
  } else if (is.character(point_dist_fn) && point_dist_fn == "n") {
    # Points will be placed using a multivariate normal distribution
    # centered at the point projection
    pt_from_proj_fn <- clupoints_n
  } else {
    stop("point_dist_fn has to be either \"n-1\", \"n\" or a user-defined ",
         "function")
  }

  # If seed was given, set it
  if (!is.na(seed)) {
    set.seed(seed)
  }

  # ############################ #
  # Determine cluster properties #
  # ############################ #

  # Normalize main direction(s)
  direction <- matrix(apply(direction, 1, function(x) x / norm(x, "2")),
                      ncol = num_dims,
                      byrow = TRUE)

  # If only one main direction was given, expand it for all clusters
  if (dim(direction)[1] == 1) {
    direction <- matrix(direction,
                        nrow = num_clusters,
                        ncol = length(direction),
                        byrow = TRUE)
  }

  # Determine cluster sizes
  cluster_sizes <- if (is.function(clusizes_fn)) {
    clusizes_fn(num_clusters, num_points, allow_empty)
  } else if (is.vector(clusizes_fn) && length(clusizes_fn) == num_clusters) {
    clusizes_fn
  } else {
    stop("`clusizes_fn` has to be either a function or a ",
         "`num_clusters`-sized vector")
  }

  # Custom clusizes_fn's are not required to obey num_points, so we update
  # it here just in case it's different from what the user specified
  num_points <- sum(cluster_sizes)

  # Determine cluster centers
  cluster_centers <- if (is.function(clucenters_fn)) {
    clucenters_fn(num_clusters, cluster_sep, cluster_offset)
  } else if (is.matrix(clucenters_fn) &&
             all(dim(clucenters_fn) == c(num_clusters, num_dims))) {
    clucenters_fn
  } else {
    stop("clucenters_fn has to be either a function or a matrix of size ",
         "`num_clusters` x `num_dims`")
  }

  # Determine length of lines supporting clusters
  cluster_lengths <- if (is.function(llengths_fn)) {
    llengths_fn(num_clusters, llength, llength_disp)
  } else if (is.vector(llengths_fn) && length(llengths_fn) == num_clusters) {
    llengths_fn
  } else {
    stop("`llengths_fn` has to be either a function or a `num_clusters`-sized",
         " vector")
  }

  # Obtain angles between main direction and cluster-supporting lines
  cluster_angles <- if (is.function(angle_deltas_fn)) {
    angle_deltas_fn(num_clusters, angle_disp)
  } else if (is.vector(angle_deltas_fn) && length(angle_deltas_fn) == num_clusters) {
    angle_deltas_fn
  } else {
    stop("`angle_deltas_fn` has to be either a function or a ",
         "`num_clusters`-sized vector")
  }

  # Determine normalized cluster directions
  cluster_directions <- matrix(mapply(rand_vector_at_angle,
                                      asplit(direction, 1),
                                      cluster_angles),
                               nrow = num_clusters,
                               byrow = TRUE)

  # ################################# #
  # Determine points for each cluster #
  # ################################# #

  # Aux. vector with cumulative sum of number of points in each cluster
  cumsum_points <- c(0, cumsum(cluster_sizes))

  # Pre-allocate data structures for holding cluster info and points

  # Cluster indices of each point
  point_clusters <- vector(mode = "integer", length = num_points)
  # Point projections on cluster-supporting lines
  point_projections <- matrix(data = 0, nrow = num_points, ncol = num_dims)
  # Final points to be generated
  points <- matrix(data = 0, nrow = num_points, ncol = num_dims)

  # Loop through clusters and create points for each one
  for (i in 1:num_clusters) {

    # Skip iteration if this cluster is empty
    if (cluster_sizes[i] == 0) {
      next
    }

    # Start and end indexes for points in current cluster
    idx_start <- cumsum_points[i] + 1
    idx_end <- cumsum_points[i + 1]

    # Update cluster indices of each point
    point_clusters[idx_start:idx_end] <- i

    # Determine distance of point projections from the center of the line
    ptproj_dist_fn_center <- pointproj_fn(cluster_lengths[i], cluster_sizes[i])

    # Determine coordinates of point projections on the line using the
    # parametric line equation (this works since cluster direction is
    # normalized)
    proj  <- points_on_line(cluster_centers[i, ],
                            cluster_directions[i, ],
                            ptproj_dist_fn_center)
    point_projections[idx_start:idx_end, ] <- proj

    # Determine points from their projections on the line
    points[idx_start:idx_end, ] <- pt_from_proj_fn(proj,
                                                   lateral_disp,
                                                   cluster_lengths[i],
                                                   cluster_directions[i, ],
                                                   cluster_centers[i, ])
  }

  # Return result
  list(points = points,
       clusters = as.factor(point_clusters),
       projections = point_projections,
       sizes = cluster_sizes,
       centers = cluster_centers,
       directions = cluster_directions,
       angles = cluster_angles,
       lengths = cluster_lengths)
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

test_that("Package linting", {
  # Don't do linting on CRAN, coverage reporting or testing on CI + Not Windows
  skip_if(is_test_mode("cran"))
  skip_on_cran()
  skip_on_covr()
  skip_if(testthat:::system_os() != "windows" && testthat:::on_ci())
  pkgpath <- if (testthat:::on_ci()) {
    Sys.getenv("GITHUB_WORKSPACE")
  } else {
    getwd()
  }
  lintr::expect_lint_free(
    path = pkgpath,
    linters = lintr::linters_with_defaults(
      # The clugen() function has a large cyclomatic complexity
      # so let's increase this a bit
      cyclocomp_linter = lintr::cyclocomp_linter(complexity_limit = 60),
      # The official line length for this package is 80, but lets
      # give it some margin; also, Windows complains about the 80
      # limit due to not recognizing some UTF-8 characters.
      line_length_linter = lintr::line_length_linter(88)
    )
  )
})

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' Get angles between average cluster direction and cluster-supporting lines
#'
#' @description
#' \loadmathjax
#' Determine the angles between the average cluster direction and the
#' cluster-supporting lines. These angles are obtained from a wrapped normal
#' distribution (\mjeqn{\mu=0}{μ=0}, \mjeqn{\sigma=}{σ=} `angle_disp` ) with
#' support in the interval \mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}.
#' Note this is different from the standard wrapped normal distribution, the
#' support of which is given by the interval
#' \mjeqn{\left[-\pi,\pi\right]}{[-π, π]}.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_clusters Number of clusters.
#' @param angle_disp Angle dispersion, in radians.
#' @return Angles between the average cluster direction and the
#' cluster-supporting lines, given in radians in the interval
#' \mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' arad <- angle_deltas(4, pi / 8) # Angle dispersion of 22.5 degrees
#' arad                            # What angles deltas did we get?
#' arad * 180 / pi                 #  Show angle deltas in degrees
angle_deltas <- function(num_clusters, angle_disp) {

  # Get random angle differences using the normal distribution
  angles <- stats::rnorm(num_clusters, sd = angle_disp)

  # Reduce angle differences to the interval [-π, π]
  angles <- atan2(sin(angles), cos(angles))

  # Make sure angle differences are within interval [-π/2, π/2]
  sapply(angles, function(a) if (abs(a) > pi / 2) a - sign(a) * pi / 2 else a)
}

#' Determine cluster centers using the uniform distribution
#'
#' @description
#' \loadmathjax
#' Determine cluster centers using the uniform distribution, taking into account
#' the number of clusters (`num_clusters`) and the average cluster separation
#' (`clu_sep`).
#'
#' More specifically, let \mjseqn{c=}`num_clusters`,
#' \mjeqn{\mathbf{s}=}{s=}`clu_sep`, \mjeqn{\mathbf{o}=}{o=}`clu_offset`,
#' \mjseqn{n=}`length(clu_sep)` (i.e., number of dimensions). Cluster centers
#' are obtained according to the following equation:
#'
#' \mjdeqn{\mathbf{C}=c\mathbf{U}\cdot\operatorname{diag}(\mathbf{s}) +
#' \mathbf{1}\,\mathbf{o}^T}{C=cU.diag(s) + 1o'}
#'
#' where \mjeqn{\mathbf{C}}{C} is the \mjeqn{c \times n}{c x n} matrix of
#' cluster centers, \mjeqn{\mathbf{U}}{U} is an \mjeqn{c \times n}{c x n} matrix
#' of random values drawn from the uniform distribution between -0.5 and 0.5,
#' and \mjeqn{\mathbf{1}}{1} is an \mjeqn{c \times 1}{c x 1} vector with all
#' entries equal to 1.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_clusters Number of clusters.
#' @param clu_sep Average cluster separation (\mjeqn{n \times 1}{n x 1} vector).
#' @param clu_offset Cluster offsets (\mjeqn{n \times 1}{n x 1} vector).
#' @return A \mjeqn{c \times n}{c x n} matrix containing the cluster centers.
#'
#' @export
#'
#' @examples
#' set.seed(321)
#' clucenters(3, c(30, 10), c(-50,50))
clucenters <- function(num_clusters, clu_sep, clu_offset) {

  # Obtain a num_clusters x num_dims matrix of uniformly distributed values
  # between -0.5 and 0.5 representing the relative cluster centers
  ctr_rel <- stats::runif(num_clusters * length(clu_sep), min = -0.5, max = 0.5)
  ctr_rel <- matrix(ctr_rel, nrow = num_clusters)

  # Convert clu_sep to diagonal matrix; if it's a one-element vector just
  # use as.matrix()
  clu_sep <- if (length(clu_sep) == 1) as.matrix(clu_sep) else diag(clu_sep)

  num_clusters * ctr_rel %*% clu_sep + rep(clu_offset, each = num_clusters)
}

#' Create points from their projections on a cluster-supporting line
#'
#' @description
#' \loadmathjax
#' Each point is placed on a hyperplane orthogonal to that line and centered at
#' the point's projection, using the normal distribution (\mjeqn{\mu=0}{μ=0},
#' \mjeqn{\sigma=}{σ=} `lat_disp` ).
#'
#' @details
#' This function's main intended use is by the main [clugen] function,
#' generating the final points when the `point_dist_fn` parameter is set to
#' `"n-1"`.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param projs Point projections on the cluster-supporting line
#' (\mjeqn{p \times n}{p x n} matrix).
#' @param lat_disp Standard deviation for the normal distribution, i.e., cluster
#' lateral dispersion.
#' @param line_len Length of cluster-supporting line (ignored).
#' @param clu_dir Direction of the cluster-supporting line.
#' @param clu_ctr Center position of the cluster-supporting line (ignored).
#' @return Generated points (\mjeqn{p \times n}{p x n} matrix).
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' ctr <- c(0, 0)
#' dir <- c(1, 0)
#' pdist <- c(-0.5, -0.2, 0.1, 0.3)
#' proj <- points_on_line(ctr, dir, pdist)
#' clupoints_n_1(proj, 0.1, NA, dir, NA)
clupoints_n_1 <- function(projs, lat_disp, line_len, clu_dir, clu_ctr) {

  # Define function to get distances from points to their projections on the
  # line (i.e., using the normal distribution)
  dist_fn <- function(clu_npoints, ldisp) stats::rnorm(clu_npoints, sd = ldisp)

  # Use clupoints_n_1_template() to do the heavy lifting
  clupoints_n_1_template(projs, lat_disp, clu_dir, dist_fn)
}

#' Create points from their projections on a cluster-supporting line
#'
#' @description
#' \loadmathjax
#' Each point is placed around its projection using the normal distribution
#' (\mjeqn{\mu=0}{μ=0}, \mjeqn{\sigma=}{σ=} `lat_disp` ).
#'
#' @details
#' This function's main intended use is by the main [clugen] function,
#' generating the final points when the `point_dist_fn` parameter is set to
#' `"n"`.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param projs Point projections on the cluster-supporting line
#' (\mjeqn{p \times n}{p x n} matrix).
#' @param lat_disp Standard deviation for the normal distribution, i.e., cluster
#' lateral dispersion.
#' @param line_len Length of cluster-supporting line (ignored).
#' @param clu_dir Direction of the cluster-supporting line.
#' @param clu_ctr Center position of the cluster-supporting line (ignored).
#' @return Generated points (\mjeqn{p \times n}{p x n} matrix).
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' ctr <- c(0, 0)
#' dir <- c(1, 0)
#' pdist <- c(-0.5, -0.2, 0.1, 0.3)
#' proj <- points_on_line(ctr, dir, pdist)
#' clupoints_n(proj, 0.01, NA, dir, NA)
clupoints_n <- function(projs, lat_disp, line_len, clu_dir, clu_ctr) {

  # Number of dimensions
  num_dims <- length(clu_dir)

  # Number of points in this cluster
  clu_num_points <- nrow(projs)

  # Get random displacement vectors for each point projection
  displ <- matrix(stats::rnorm(clu_num_points * num_dims, sd = lat_disp),
                  nrow = clu_num_points)

  # Add displacement vectors to each point projection and return the result
  projs + displ
}

#' Determine cluster sizes, i.e., the number of points in each cluster
#'
#' @description
#' \loadmathjax
#' Cluster sizes are determined using the normal distribution
#' (\mjeqn{\mu=}{μ=} `num_points` \mjseqn{/} `num_clusters`,
#' \mjeqn{\sigma=\mu/3}{σ=μ/3}), and then assuring that the final cluster sizes
#' add up to `num_points` via the [fix_num_points] function.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_clusters Number of clusters.
#' @param num_points Total number of points.
#' @param allow_empty Allow empty clusters?
#' @return Number of points in each cluster (vector of length `num_clusters`).
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' sizes <- clusizes(4, 1000, TRUE)
#' sizes
#' sum(sizes)
clusizes <- function(num_clusters, num_points, allow_empty) {

    # Determine number of points in each cluster using the normal distribution

    # Consider the mean an equal division of points between clusters
    mean <- num_points / num_clusters
    # The standard deviation is such that the interval [0, 2 * mean] will
    # contain ≈99.7% of cluster sizes
    std <- mean / 3

    # Determine points with the normal distribution
    clu_num_points <- stats::rnorm(num_clusters, mean = mean, sd = std)

    # Set negative values to zero
    clu_num_points <- sapply(clu_num_points, function(x) if (x > 0) x else 0)

    # Fix imbalances, so that num_points is respected
    if (sum(clu_num_points) > 0) { # Be careful not to divide by zero
      clu_num_points <- num_points / sum(clu_num_points) * clu_num_points
    }

    # Round the real values to integers since a cluster sizes is represented by
    # an integer
    clu_num_points <- as.integer(round(clu_num_points))

    # Make sure total points is respected, which may not be the case at this
    # time due to rounding
    clu_num_points <- fix_num_points(clu_num_points, num_points)

    # If empty clusters are not allowed, make sure there aren't any
    if (!allow_empty) {
      clu_num_points <- fix_empty(clu_num_points, allow_empty)
    }

    clu_num_points
}

#' Determine length of cluster-supporting lines
#'
#' @description
#' \loadmathjax
#' Line lengths are determined using the folded normal distribution
#' (\mjeqn{\mu=}{μ=} `llength`, \mjeqn{\sigma=}{σ=} `llength_disp` ).
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_clusters Number of clusters.
#' @param llength Average line length.
#' @param llength_disp Line length dispersion.
#' @return Lengths of cluster-supporting lines (vector of size `num_clusters`).
#'
#' @export
#'
#' @examples
#' set.seed(123)
#' llengths(4, 20, 3.5)
llengths <- function(num_clusters, llength, llength_disp) {
  abs(stats::rnorm(num_clusters, mean = llength, sd = llength_disp))
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nclu = num_clusters, astd = angles_stds)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nclu <- targs[i, "nclu"]
  astd <- targs[i, "astd"]

  # Set seed
  set.seed(seed)

  # Determine test name for current parameter set
  test_desc <- paste0("angle_deltas: seed=", seed, ",
                      nclu=", nclu, ", astd=", astd)

  # Perform tests for current parameter set
  test_that(test_desc, {

    # Check that the angle_deltas function runs without warnings
    expect_warning(angles <- angle_deltas(nclu, astd), regexp = NA)

    # Check that return value has the correct dimensions
    expect_equal(length(angles), nclu)

    # Check that all angles are between -π/2 and π/2
    expect_true(all(abs(angles) <= pi / 2))
  })
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims, nclu = num_clusters)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]
  nclu <- targs[i, "nclu"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (cluster offsets + seps)
  tsargs <- expand.grid(cluoff = asplit(get_clu_offsets(nd), 1),
                        clusep = asplit(get_clu_seps(nd), 1))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current line direction + line center
    cluoff <- tsargs[j, "cluoff"][[1]]
    clusep <- tsargs[j, "clusep"][[1]]

    # Determine test name for current parameter set
    test_desc <- paste0("clucenters: nd=", nd, ", nclu=", nclu,
                        ", cluoff=[", paste(cluoff, collapse = ", "),
                        "], clusep=[", paste(clusep, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # Check that the clucenters function runs without warnings
      expect_warning(clu_ctrs <- clucenters(nclu, clusep, cluoff), regexp = NA)

      # Check that return value has the correct dimensions
      expect_equal(dim(clu_ctrs), c(nclu, nd))
    })
  }
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nclu = num_clusters, tpts = num_points,
                     ae = allow_empties)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nclu <- targs[i, "nclu"]
  tpts <- targs[i, "tpts"]
  ae <- targs[i, "ae"]

  # Don't test if number of points is less than number of clusters and we don't
  # allow empty clusters
  if (!ae && tpts < nclu) {
    next
  }

  # Set seed
  set.seed(seed)

  # Determine test name for current parameter set
  test_desc <- paste0("clusizes: nclu=", nclu, ", tpts=", tpts, ", ae=", ae)

  # Perform tests for current parameter set
  test_that(test_desc, {

    # Check that the function runs without warnings
    expect_warning(clu_sizes <- clusizes(nclu, tpts, ae), regexp = NA)

    # Check that the output has the correct number of clusters
    expect_equal(length(clu_sizes), nclu)

    # Check that the total number of points is correct
    expect_equal(sum(clu_sizes), tpts)

    # # If empty clusters are not allowed, check that all of them have points
    if (!ae) {
      expect_true(min(clu_sizes) > 0)
    }
  })
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nclu = num_clusters, len = llengths_mus,
                     lensd = llengths_sigmas)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nclu <- targs[i, "nclu"]
  len <- targs[i, "len"]
  lensd <- targs[i, "lensd"]

  # Set seed
  set.seed(seed)

  # Determine test name for current parameter set
  test_desc <- paste0("llengths: nclu=", nclu, ", len=", len, ", lensd=", lensd)

  # Perform tests for current parameter set
  test_that(test_desc, {

    # Check that the function runs without warnings
    expect_warning(lens <- llengths(nclu, len, lensd), regexp = NA)

    # Check that the output has the correct number of clusters
    expect_equal(length(lens), nclu)

    # Check that all lengths are >= 0
    expect_true(min(lens) >= 0)
  })
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

library(ggplot2)   # For plotting in 2D
library(patchwork) # For combining 2D plots

# Function for plotting the 1D clusters, showing density
plot_examples_1d <- function(..., pmargin = 0.1, ymax = 0.6) {

  # Place examples in a list
  ets <- list(...)

  # Get maximum and minimum points in each dimension for all examples
  xmax <- max(sapply(ets, function(et) max(et$e$points)))
  xmin <- min(sapply(ets, function(et) min(et$e$points)))

  # Determine plot margins beyond maximum and minimum points
  xd <- pmargin * abs(xmax - xmin)
  xmax <- xmax + xd
  xmin <- xmin - xd

  # Create plots
  plts <- lapply(
    ets,
    function(et) {
      e <- et$e
      t <- et$t
      ggplot() +
        geom_density(mapping = aes(x = e$points,
                                   colour = e$clusters,
                                   fill = e$clusters),
                     alpha = 0.3) +
        geom_point(mapping = aes(x = e$points,
                                 y = -0.02,
                                 fill = e$clusters),
                   shape = 21,
                   colour = "black",
                   stroke = 0.1,
                   alpha = 0.2) +
        ggtitle(t) + xlab(NULL) + ylab(NULL) +
        xlim(xmin, xmax) + ylim(-0.025, ymax) +
        theme(legend.position = "none",
              plot.title = element_text(size = rel(0.75)),
              axis.ticks.y = element_blank(),
              axis.text.y = element_blank()
        )
    })

  # Combine plots as subplots
  wrap_plots(plts)
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' Determine coordinates of points on a line
#'
#' @description
#' \loadmathjax
#' Determine coordinates of points on a line with `center` and `direction`,
#' based on the distances from the center given in `dist_center`.
#'
#' This works by using the vector formulation of the line equation assuming
#' `direction` is a \mjseqn{n}-dimensional unit vector. In other words,
#' considering \mjeqn{\mathbf{d}=}{d =} `as.matrix(direction)` (\mjeqn{n \times
#' 1}{n x 1} vector), \mjeqn{\mathbf{c}=}{c =} `as.matrix(center)` (\mjeqn{n
#' \times 1}{n x 1} vector), and \mjeqn{\mathbf{w}=}{w =}
#' `as.matrix(dist_center)` (\mjeqn{p \times 1}{p x 1} vector), the coordinates
#' of points on the line are given by:
#'
#' \mjdeqn{\mathbf{P}=\mathbf{1}\,\mathbf{c}^T + \mathbf{w}\mathbf{d}^T}{
#' P = 1c' + wd'}
#'
#' where \mjeqn{\mathbf{P}}{P} is the \mjeqn{p \times n}{p x n} matrix of point
#' coordinates on the line, and \mjeqn{\mathbf{1}}{1} is a \mjeqn{p \times 1}{p
#' x 1} vector with all entries equal to 1.
#'
#' @param center Center of the line (\mjseqn{n}-component vector).
#' @param direction Line direction (\mjseqn{n}-component unit vector).
#' @param dist_center  Distance of each point to the center of the line
#' (\mjseqn{n}-component vector, where \mjseqn{n} is the number of points).
#' @return Coordinates of points on the specified line
#' (\mjeqn{p \times n}{p x n} matrix).
#'
#' @export
#'
#' @examples
#' points_on_line(c(5, 5), c(1, 0), seq(-4, 4, length.out=5)) # 2D, 5 points
#'
#' points_on_line(c(-2, 0, 0, 2), c(0, 0, -1, 0), c(10, -10)) # 4D, 2 points
points_on_line <- function(center, direction, dist_center) {
  rep(center, each = length(dist_center)) +
    dist_center %*% matrix(direction, nrow = 1)
}

#' Get a random unit vector orthogonal to `u`.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param u A unit vector.
#' @return A random unit vector orthogonal to `u`.
#'
#' @export
#'
#' @examples
#' r <- stats::runif(3)      # Get a random 3D vector
#' r <- r / norm(r, "2")     # Normalize it
#' o <- rand_ortho_vector(r) # Get a random unit vector orthogonal to r
#' r %*% o            # Check that r and o are orthogonal (result should be ~0)
rand_ortho_vector <- function(u) {

  if (length(u) == 1) {

    # If 1D, just return a random unit vector
    v <- rand_unit_vector(1)

  } else {

    # Otherwise find a random, non-parallel vector to u
    while (TRUE) {

      # Find normalized random vector
      r <- rand_unit_vector(length(u))

      # If not parallel to u we can keep it and break the loop
      if (!isTRUE(all.equal(abs(u %*% r), 1))) {
        break
      }
    }

    # Get vector orthogonal to u using 1st iteration of Gram-Schmidt process
    v <- r - c(u %*% r) / c(u %*% u) * u

    # Normalize it
    v <- v / norm(v, "2")
  }

  # Return it
  v
}

#' Get a random unit vector with `num_dims` components.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param num_dims Number of components in vector (i.e. vector size).
#' @return A random unit vector with `num_dims` components.
#'
#' @export
#'
#' @examples
#' r <- rand_unit_vector(4)
#' norm(r, "2")
rand_unit_vector <- function(num_dims) {
  r <- stats::runif(num_dims, min = -0.5, max = 0.5)
  r / norm(r, "2")
}

#' Get a random unit vector at a given angle with another vector.
#'
#' @description
#' \loadmathjax
#' Get a random unit vector which is at `angle` radians of vector `u`.
#' Note that `u` is expected to be a unit vector itself.
#'
#' @note This function is stochastic. For reproducibility set a PRNG seed with
#' [set.seed].
#'
#' @param u Unit vector with \mjseqn{n} components.
#' @param angle Angle in radians.
#' @return Random unit vector with \mjseqn{n} components which is at `angle`
#' radians with vector `u`.
#'
#' @export
#'
#' @examples
#' u <- c(1.0, 0, 0.5, -0.5)            # Define a 4D vector
#' u <- u / norm(u, "2")                # Normalize the vector
#' v <- rand_vector_at_angle(u, pi / 4) # Get a vector at 45 degrees
#' arad <- acos((u %*% v) / norm(u,"2") * norm(v, "2")) # Get angle in radians
#' arad * 180 / pi # Convert to degrees, should be close to 45 degrees
rand_vector_at_angle <- function(u, angle) {
  if (isTRUE(all.equal(angle, 0))) {
    v <- u
  } else if (isTRUE(all.equal(abs(angle), pi / 2)) && length(u) > 1) {
    v <- rand_ortho_vector(u)
  } else if ((abs(angle) < pi / 2) && length(u) > 1) {
    v <- u + rand_ortho_vector(u) * tan(angle)
    v <- v / norm(v, "2")
  } else {
    # For |θ| > π/2 or the 1D case, simply return a random vector
    v <- rand_unit_vector(length(u))
  }
  v
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]

  # Set seed
  set.seed(seed)

  # Go through a number of randomly created unit vectors
  for (u in asplit(get_unitvecs(nvec, nd), 1)) {

    # Determine test name for current parameter set
    test_desc <- paste0("rand_ortho_vector: nd=", nd,
                        ", u=[", paste(u, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # Check that the function runs without warnings
      expect_warning(r <- rand_ortho_vector(u), regexp = NA)

      # Check that returned vector has the correct length
      expect_equal(length(r), nd)

      # Check that returned vector has norm == 1
      expect_equal(norm(r, "2"), 1)

      # Check that vectors u and r are orthogonal (only for nd > 1)
      if (nd > 1) {
        # The dot product of orthogonal vectors must be (approximately) zero
        expect_equal((u %*% r)[1], 0)
      }
    })
  }
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]

  # Set seed
  set.seed(seed)

  # Determine test name for current parameter set
  test_desc <- paste0("rand_unit_vector: nd=", nd)

  # Perform tests for current parameter set
  test_that(test_desc, {

    # Check that the function runs without warnings
    expect_warning(r <- rand_unit_vector(nd), regexp = NA)

    # Check that returned vector has the correct length
    expect_equal(length(r), nd)

    # Check that returned vector has norm == 1
    expect_equal(norm(r, "2"), 1)
  })
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

is_test_mode <- function(tm) {
  identical(tolower(Sys.getenv("CLUGENR_TEST_MODE")), tolower(tm))
}

if (testthat:::on_cran() || is_test_mode("cran")) {
  # Light tests, for CRAN
  cat("Testing on CRAN\n")

  # Parameters to sweep
  seeds <- c(123)
  num_dims <- c(2)
  num_points <- c(10)
  num_clusters <- c(3)
  lat_stds <- c(10)
  llengths_mus <- c(1)
  llengths_sigmas <- c(1)
  angles_stds <- c(pi / 512)
  allow_empties <- c(TRUE)

  # Number of line directions to test
  ndirs <- 1

  # Number of line centers to test
  ncts <- 1

  # How many vectors to test?
  nvec <- 1

  # How many angles to test?
  nang <- 1

} else if (testthat:::on_ci() || is_test_mode("ci")) {
  # Test setup for CI environments
  cat("Testing on CI\n")

  # Parameters to sweep
  seeds <- c(321)
  num_dims <- c(1, 3, 10)
  num_points <- c(1, 200)
  num_clusters <- c(1, 10)
  lat_stds <- c(0.0, 5.0)
  llengths_mus <- c(0, 10)
  llengths_sigmas <- c(0, 15)
  angles_stds <- c(0, pi / 256, pi / 2, pi)
  allow_empties <- c(TRUE, FALSE)

  # Number of line directions to test
  ndirs <- 1

  # Number of line centers to test
  ncts <- 1

  # How many vectors to test?
  nvec <- 2

  # How many angles to test?
  nang <- 2

} else if (!is_test_mode("full")) {
  # Test setup for local tests
  cat("Testing locally\n")

  # Parameters to sweep
  seeds <- c(0, 123)
  num_dims <- c(1, 2, 3, 4, 30)
  num_points <- c(1, 10, 500)
  num_clusters <- c(1, 5, 20, 50)
  lat_stds <- c(0.0, 5.0)
  llengths_mus <- c(0, 10)
  llengths_sigmas <- c(0, 15)
  angles_stds <- c(0, pi / 4, pi / 2, pi)
  allow_empties <- c(TRUE, FALSE)

  # Number of line directions to test
  ndirs <- 2

  # Number of line centers to test
  ncts <- 2

  # How many vectors to test?
  nvec <- 4

  # How many angles to test?
  nang <- 4

} else {
  # Heavy-duty tests if the env variable CLUGENR_TEST_FULL is set to "true"
  # Can take a long time
  cat("Testing exhaustively (slow!)\n")

  # Parameters to sweep
  seeds <- c(0, 123)
  num_dims <- c(1, 2, 3, 4, 30)
  num_points <- c(1, 10, 500, 2000, 10000)
  num_clusters <- c(1, 2, 5, 10, 100)
  lat_stds <- c(0.0, 5.0, 500)
  llengths_mus <- c(0, 10)
  llengths_sigmas <- c(0, 15)
  angles_stds <- c(0, pi / 256, pi / 4, pi / 2, pi, 2 * pi)
  allow_empties <- c(TRUE, FALSE)

  # Number of line directions to test
  ndirs <- 4

  # Number of line centers to test
  ncts <- 4

  # How many vectors to test?
  nvec <- 5

  # How many angles to test?
  nang <- 5
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

library(ggplot2)   # For plotting in 2D
library(patchwork) # For combining 2D plots

# Function for plotting the nD clusters
plot_examples_nd <- function(e, title, pstroke = 0.05, psize = 0.75,
                             pmargin = 0.1) {

  # How many dimensions?
  nd <- ncol(e$points)

  # Get maximum and minimum points in each dimension
  xmaxs <- apply(e$points, 2, max)
  xmins <- apply(e$points, 2, min)

  # Determine plots centers in each dimension
  xcenters <- (xmaxs + xmins) / 2

  # Determine plot span in all dimensions
  sidespan <- (1 + pmargin) * max(abs(xmaxs - xmins)) / 2

  # Determine final plots limits in all dimensions
  xmaxs <- xcenters + sidespan
  xmins <- xcenters - sidespan

  # All possible combinations
  idxs <- expand.grid(1:nd, 1:nd)

  # Create plots
  plts <- apply(idxs,
                1,
                function(x) {
                  if (x[1] == x[2]) {
                    grid::textGrob(paste0("x", x[1]))
                  } else {
                    ggplot(NULL,
                           aes(x = e$points[, x[1]],
                               y = e$points[, x[2]],
                               colour = e$clusters)) +
                      geom_point(shape = 21,
                                 colour = "black",
                                 stroke = pstroke,
                                 size = psize,
                                 aes(fill = e$clusters)) +
                      xlab(NULL) + ylab(NULL) +
                      theme(legend.position = "none",
                            axis.ticks.x = element_blank(),
                            axis.text.x = element_blank(),
                            axis.ticks.y = element_blank(),
                            axis.text.y = element_blank()) +
                      coord_fixed(xlim = c(xmins[x[1]], xmaxs[x[1]]),
                                  ylim = c(xmins[x[2]], xmaxs[x[2]]))
                  }
                })

  # Combine plots as subplots
  wrap_plots(plts) + plot_annotation(
    title = title,
    theme = theme(plot.title = element_text(size = rel(0.8))))
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Get n random vectors with nd-dimensions
get_vecs <- function(n, nd) {
  matrix(runif(n * nd), ncol = nd)
}

# Get n random unit vectors with nd-dimensions
get_unitvecs <- function(n, nd) {
  m <- matrix(runif(n * nd), ncol = nd)
  for (i in 1:n) {
    m[i, ] <- m[i, ] / norm(as.matrix(m[i, ]), type = "2")
  }
  m
}

# Get n angles
get_angles <- function(n) {
  stats::runif(n, min = -pi, max = pi)
}

# Get cluster offsets
get_clu_offsets <- function(nd) {
  clo <- matrix(0, 3, nd)
  clo[2, ] <- 1
  clo[3, ] <- stats::rnorm(nd, sd = 1000)
  clo
}

# Get cluster separators
get_clu_seps <- get_clu_offsets

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims, len = llengths_mus,
                     tpts = num_points)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  tpts <- targs[i, "tpts"]
  nd <- targs[i, "nd"]
  len <- targs[i, "len"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (line directions + centers)
  tsargs <- expand.grid(direc = asplit(get_unitvecs(ndirs, nd), 1),
                        ctr = asplit(get_vecs(ncts, nd), 1))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current line direction + line center
    direc <- tsargs[j, "direc"][[1]]
    ctr <- tsargs[j, "ctr"][[1]]

    # Determine test name for current parameter set
    test_desc <- paste0("points_on_line: nd=", nd,
                        ", len=", len, ", tpts=", tpts,
                        ", direc=[", paste(direc, collapse = ", "),
                        "], ctr=[", paste(ctr, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # Create some random distances from center
      dist2ctr <- stats::runif(tpts, min = -len / 2, max = len / 2)

      # Check that the points_on_line function runs without warnings
      expect_warning(pts <- points_on_line(ctr, direc, dist2ctr),
                     regexp = NA)

      # Check that the dimensions agree
      expect_equal(dim(pts), c(tpts, nd))

      # Check that distance of points to the line is approximately zero
      for (pt in asplit(pts, 1)) {
        # Get distance from current point to line
        dp2l <- (pt - ctr) - c((pt - ctr) %*% direc) * direc
        d <- norm(as.matrix(dp2l, type = "2"))
        # Check that it is approximately zero
        expect_equal(d, 0)
      }
    })
  }
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims, len = llengths_mus,
                     tpts = num_points, lat_std = lat_stds)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  tpts <- targs[i, "tpts"]
  nd <- targs[i, "nd"]
  len <- targs[i, "len"]
  lat_std <- targs[i, "lat_std"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (line directions + centers)
  tsargs <- expand.grid(direc = asplit(get_unitvecs(ndirs, nd), 1),
                        ctr = asplit(get_vecs(ncts, nd), 1))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current line direction + line center
    direc <- tsargs[j, "direc"][[1]]
    ctr <- tsargs[j, "ctr"][[1]]

    # Determine test name for current parameter set
    test_desc <- paste0("clupoints_n_1: nd=", nd,
                        ", len=", len, ", tpts=", tpts, ", lat_std=", lat_std,
                        ", direc=[", paste(direc, collapse = ", "),
                        "], ctr=[", paste(ctr, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # Create some random distances from center
      proj_dist_fn2ctr <- stats::runif(tpts, min =  -len / 2, max = len / 2)
      proj <- points_on_line(ctr, direc, proj_dist_fn2ctr)

      # Check that the clupoints_n_1 function runs without warnings
      expect_warning(
        pts <- clupoints_n_1(proj, lat_std, len, direc, ctr),
        regexp = NA)

      # Check that number of points is the same as the number of projections
      expect_equal(dim(pts), dim(proj))

      # The following checks are only for dimensionality above 1
      if (nd > 1) {
        # For each vector from projection to point...
        for (u in asplit(pts - proj, 1)) {
          # Vector should be approximately orthogonal to the cluster line
          expect_equal((direc %*% u)[1], 0)
        }
      }
    })
  }
}

# Copyright (c) 2020-2023 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

# Create parameter combinations to test
targs <- expand.grid(seed = seeds, nd = num_dims)

# Loop through all parameter combinations
for (i in seq.int(1, nrow(targs))) {

  # Get current parameters
  seed <- targs[i, "seed"]
  nd <- targs[i, "nd"]

  # Set seed
  set.seed(seed)

  # Create combination of seed-depending parameters (line directions + centers)
  tsargs <- expand.grid(u = asplit(get_unitvecs(nvec, nd), 1),
                        a = c(get_angles(nang), pi / 2))

  # Loop through line directions and line centers
  for (j in seq.int(1, nrow(tsargs))) {

    # Get current vector and angle
    u <- tsargs[j, "u"][[1]]
    a <- tsargs[j, "a"]

    # Determine test name for current parameter set
    test_desc <- paste0("rand_vector_at_angle: nd=", nd, ", a=", a,
                        ", u=[", paste(u, collapse = ", "), "]")

    # Perform tests for current parameter set
    test_that(test_desc, {

      # Check that the function runs without warnings
      expect_warning(r <- rand_vector_at_angle(u, a), regexp = NA)

      # Check that returned vector has the correct length
      expect_equal(length(r), nd)

      # Check that returned vector has norm == 1
      expect_equal(norm(r, "2"), 1)

      # Check that vectors u and r have an angle of a between them
      if (nd > 1 && abs(a) < pi / 2) {
        expect_equal(angle_btw(u, r), abs(a))
      }

      # Check corner case where angle == pi / 2 and vector length > 1
      if (nd > 1 && a == pi / 2) {
        v <- stats::runif(1, 2, 100) * u
        expect_warning(r <- rand_vector_at_angle(v, a), regexp = NA)
        expect_equal(angle_btw(v, r), pi / 2)
      }
    })
  }
}

# Copyright (c) 2020-2022 Nuno Fachada
# Distributed under the MIT License (http://opensource.org/licenses/MIT)

#' clugenr: Multidimensional cluster generation in R
#'
#' clugenr is an R package for generating multidimensional clusters. Each
#' cluster is supported by a line segment, the position, orientation and length
#' of which guide where the respective points are placed.
#'
#' @author Nuno Fachada
#'
#' @note MIT License
#'
#' @docType package
#' @name clugenr
# Suppress R CMD check note
#' @importFrom mathjaxr preview_rd
NULL
#> NULL

---
title: "Examples in 2D"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Examples in 2D}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{ggplot2,patchwork}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.5,
  dpi = 120
)
```

This vignette contains a number of examples on how to use `clugenr` in 2D.
Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
# Load the clugenr library
library(clugenr)

# Load functions for plotting examples
source("plot_examples_2d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 2D examples were plotted with the `plot_examples_2d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_2d.R).

## Manipulating the direction of cluster-supporting lines

### Using the `direction` parameter

```{r}
seed <- 123
```

```{r}
e001 <- clugen(2, 4, 200, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e002 <- clugen(2, 4, 200, c(1, 1), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e003 <- clugen(2, 4, 200, c(0, 1), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
```

```{r}
plot_examples_2d(list(e = e001, t = "e001: direction = [1, 0]"),
                 list(e = e002, t = "e002: direction = [1, 1]"),
                 list(e = e003, t = "e003: direction = [0, 1]"))
```

### Changing the `angle_disp` parameter and using a custom `angle_deltas_fn` function

```{r}
seed <- 123
```

```{r}
# Custom angle_deltas function: arbitrarily rotate some clusters by 90 degrees
angdel_90_fn <- function(nclu, astd) sample(c(0, pi / 2), nclu, replace = TRUE)
```

```{r}
e004 <- clugen(2, 6, 500, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e005 <- clugen(2, 6, 500, c(1, 0), pi / 8, c(10, 10), 10, 1.5, 0.5, seed = seed)
e006 <- clugen(2, 6, 500, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed,
               angle_deltas_fn = angdel_90_fn)
```

```{r}
plot_examples_2d(list(e = e004, t = "e004: angle_disp = 0"),
                 list(e = e005, t = "e005: angle_disp = π/8"),
                 list(e = e006, t = "e006: custom angle_deltas function"))
```

## Manipulating the length of cluster-supporting lines

### Using the `llength` parameter

```{r}
seed <- 321
```

```{r}
e007 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 0, 0, 0.5, seed = seed,
               point_dist_fn = "n")
e008 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 10, 0, 0.5, seed = seed,
               point_dist_fn = "n")
e009 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 30, 0, 0.5, seed = seed,
               point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e007, t = "e007: llength = 0"),
                 list(e = e008, t = "e008: llength = 10"),
                 list(e = e009, t = "e009: llength = 30"))
```

### Changing the `llength_disp` parameter and using a custom `llengths_fn` function

```{r}
seed <- 765
```

```{r}
# Custom llengths function: line lengths grow for each new cluster
llen_grow_fn <- function(nclu, llen, llenstd) {
  llen * 0:(nclu - 1) + rnorm(nclu, sd = llenstd)
}
```

```{r}
e010 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 15,  0.0, 0.5,
               seed = seed, point_dist_fn = "n")
e011 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 15, 10.0, 0.5,
               seed = seed, point_dist_fn = "n")
e012 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 10,  0.1, 0.5,
               seed = seed, llengths_fn = llen_grow_fn, point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e010, t = "e010: llength_disp = 0.0"),
                 list(e = e011, t = "e011: llength_disp = 5.0"),
                 list(e = e012, t = "e012: custom llengths function"))
```

## Manipulating relative cluster positions

### Using the `cluster_sep` parameter

```{r}
seed <- 222
```

```{r}
e013 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 15, 2, 4, seed = seed)
e014 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(30, 10), 15, 2, 4, seed = seed)
e015 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 30), 15, 2, 4, seed = seed)
```

```{r}
plot_examples_2d(list(e = e013, t = "e013: cluster_sep = [10, 10]"),
                 list(e = e014, t = "e014: cluster_sep = [30, 10]"),
                 list(e = e015, t = "e015: cluster_sep = [10, 30]"))
```

### Changing the `cluster_offset` parameter and using a custom `clucenters_fn` function

```{r}
seed <- 222
```

```{r}
# Custom clucenters function: places clusters in a diagonal
centers_diag <- function(nclu, csep, coff) {
  matrix(1, nrow = nclu, ncol = length(csep)) * (1:nclu * max(csep)) +
    rep(coff, each = nclu)
}
```

```{r}
e016 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed)
e017 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed,
               cluster_offset = c(20, -20))
e018 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed,
               cluster_offset = c(-50, -50), clucenters_fn = centers_diag)
```

```{r}
plot_examples_2d(list(e = e016, t = "e016: default"),
                 list(e = e017, t = "e017: cluster_offset = [20, -20]"),
                 list(e = e018, t = "e018: custom clucenters function"))
```

## Lateral dispersion and placement of point projections on the line

```{r}
seed <- 234
```

### Normal projection placement (default): `proj_dist_fn = "norm"`

```{r}
e019 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed)
e020 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed)
e021 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed)
```

```{r}
plot_examples_2d(list(e = e019, t = "e019: lateral_disp = 0"),
                 list(e = e020, t = "e020: lateral_disp = 1"),
                 list(e = e021, t = "e021: lateral_disp = 3"))
```

### Uniform projection placement: `proj_dist_fn = "unif"`

```{r}
e022 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = "unif")
e023 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = "unif")
e024 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = "unif")
```

```{r}
plot_examples_2d(list(e = e022, t = "e022: lateral_disp = 0"),
                 list(e = e023, t = "e023: lateral_disp = 1"),
                 list(e = e024, t = "e024: lateral_disp = 3"))
```

### Custom projection placement using the Beta distribution

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

```{r}
e025 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = proj_beta)
e026 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = proj_beta)
e027 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e025, t = "e025: lateral_disp = 0"),
                 list(e = e026, t = "e026: lateral_disp = 1"),
                 list(e = e027, t = "e027: lateral_disp = 3"))
```

## Controlling final point positions from their projections on the cluster-supporting line

```{r}
seed <- 12321
```

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

### Points on hyperplane orthogonal to cluster-supporting line (default): `point_dist_fn = "n-1"`

```{r}
e028 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed)
e029 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               proj_dist_fn = "unif")
e030 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e028, t = "e028: proj_dist_fn = 'norm' (default)"),
                 list(e = e029, t = "e029: proj_dist_fn = 'unif'"),
                 list(e = e030, t = "e030: custom proj_dist_fn (Beta dist.)"))
```

### Points around projection on cluster-supporting line: `point_dist_fn = "n"`

```{r}
e031 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n")
e032 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n", proj_dist_fn = "unif")
e033 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n", proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e031, t = "e031: proj_dist_fn = 'norm' (default)"),
                 list(e = e032, t = "e032: proj_dist_fn = 'unif'"),
                 list(e = e033, t = "e033: custom proj_dist_fn (Beta dist.)"))
```

### Custom point placement using the exponential distribution

```{r}
# Custom point_dist_fn: final points placed using the Exponential distribution
clupoints_n_1_exp <- function(projs, lat_std, len, clu_dir, clu_ctr) {
  dist_exp <- function(npts, lstd) lstd * rexp(npts, rate = 2 / lstd)
  clupoints_n_1_template(projs, lat_std, clu_dir, dist_exp)
}
```

```{r}
e034 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp)
e035 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = "unif")
e036 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e034, t = "e034: proj_dist_fn = 'norm' (default)"),
                 list(e = e035, t = "e035: proj_dist_fn = 'unif'"),
                 list(e = e036, t = "e036: custom proj_dist_fn (Beta dist.)"))
```

## Manipulating cluster sizes

```{r}
seed <- 87
```

```{r}
# Custom clucenters_fn (all): yields fixed positions for the clusters
centers_fixed <- function(nclu, csep, coff) {
  matrix(c(-csep[1], -csep[2], csep[1], -csep[2], -csep[1], csep[2], csep[1], csep[2]),
         nrow = nclu, byrow = TRUE)
}

# Custom clusizes_fn (e038): cluster sizes determined via the uniform distribution,
# no correction for total points
clusizes_unif <- function(nclu, npts, ae) sample(2 * npts / nclu, nclu, replace = TRUE)

# Custom clusizes_fn (e039): clusters all have the same size, no correction for
# total points
clusizes_equal <- function(nclu, npts, ae) npts %/% nclu * rep.int(1, nclu)
```

```{r}
e037 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, point_dist_fn = "n")
e038 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, clusizes_fn = clusizes_unif,
               point_dist_fn = "n")
e039 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, clusizes_fn = clusizes_equal,
               point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e037, t = "e037: normal dist. (default)"),
                 list(e = e038, t = "e038: unif. dist. (custom)"),
                 list(e = e039, t = "e039: equal size (custom)"))
```

## Direct specification of optional parameters

```{r}
seed <- 9876
```

```{r}
e040 <- clugen(2, 4, 1000, c(-1, 1), 0, c(0, 0), 0, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(50, 200, 500, 2000), llengths_fn = c(0, 2, 4, 6),
               clucenters_fn = matrix(c(-5, -5, -2.5, -2.5, 0, 0, 2.5, 2.5),
                                      nrow = 4, byrow = TRUE))
e041 <- clugen(2, 5, 1000, matrix(c(1, 1, 1, 0, 1, 0, 0, 1, 0, 1),
                                  nrow = 5, byrow = TRUE),
               0, c(0, 0), 0, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(200, 500, 500, 500, 500),
               llengths_fn = c(0, 5, 5, 5, 5),
               clucenters_fn = matrix(c(0, 0, 0, 5, 0, -5, 5, 0, -5, 0),
                                      nrow = 5, byrow = TRUE))
e042 <- clugen(2, 5, 1000, matrix(c(0, 1, 0.25, 0.75, 0.5, 0.5, 0.75, 0.25, 1, 0),
                                  nrow = 5, byrow = TRUE),
               0, c(0, 0), 5, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(500, 500, 500, 500, 500),
               clucenters_fn = matrix(c(-5, 0, -3, -0.3, -1, -0.8, 1, -1.6, 3, -2.5),
                                      nrow = 5, byrow = TRUE))
```

```{r}
plot_examples_2d(list(e = e040, t = "e040: direct params 1"),
                 list(e = e041, t = "e041: direct params 2"),
                 list(e = e042, t = "e042: direct params 3"))
```

---
title: "Examples in other dimensions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples in other dimensions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{ggplot2,patchwork}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.5,
  dpi = 200
)
```

This vignette contains a number of examples on how to use `clugenr` in 1D and
5D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
# Load the clugenr library
library(clugenr)

# Load functions for plotting examples
source("plot_examples_1d.R", local = knitr::knit_global())
source("plot_examples_nd.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

## Basic 1D example with density plot

The 1D examples were plotted with the `plot_examples_1d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_1d.R).

```{r}
seed <- 222
```

```{r}
# Custom proj_dist_fn: point projections placed using the Weibull distribution
proj_wbull <- function(len, n) rweibull(n, shape = 1.5, scale = len / 2) - len / 2
```

```{r}
e085 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed)
e086 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed, proj_dist_fn = "unif")
e087 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed, proj_dist_fn = proj_wbull)
```

```{r}
plot_examples_1d(list(e = e085, t = "e085: proj_dist_fn='norm'"),
                 list(e = e086, t = "e086: proj_dist_fn='unif'"),
                 list(e = e087, t = "e087: custom proj_dist_fn (Weibull)"),
                 pmargin = 0.07,
                 ymax = 0.575)
```

## 5D example with default optional arguments

The 5D examples were plotted with the `plot_examples_nd()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_nd.R).

```{r}
nd <- 5
seed <- 321
```

```{r}
e088 <- clugen(nd, 6, 1500, c(1, 1, 0.5, 0, 0), pi / 16, rep.int(30, nd), 30, 4, 3,
               seed = seed)
```

```{r, fig.asp = 1}
plot_examples_nd(e088, "e088: 5D with optional parameters set to defaults")
```

## 5D example with `proj_dist_fn = "unif"` and `point_dist_fn = "n"`

```{r}
nd <- 5
seed <- 123
```

```{r}
e089 <- clugen(nd, 6, 1500, c(0.1, 0.3, 0.5, 0.3, 0.1), pi / 12, rep.int(30, nd),
               35, 5, 3.5, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n")
```

```{r, fig.asp = 1}
plot_examples_nd(e089, "e089: 5D with proj_dist_fn='unif' and point_dist_fn='n'")
```

---
title: "Examples in 3D: part II"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Examples in 3D: part II}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{rgl}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.4,
  fig.width = 8.766
)
```

This vignette contains part II of a set of examples on how to use `clugenr` in
3D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
library(clugenr)    # The clugenr library
options(rgl.useNULL = TRUE)  # Create RGL plots in systems without displays (CI)
library(rgl)
setupKnitr(autoprint = TRUE) # Render RGL plots directly on generated page

# Load helper functions for plotting examples
source("plot_examples_3d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 3D examples were plotted with the `plot_examples_3d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_3d.R).

## Lateral dispersion and placement of point projections on the line

```{r}
seed <- 456
```

### Normal projection placement (default): `proj_dist_fn = "norm"`

```{r}
e064 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed)
e065 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed)
e066 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed)
```

```{r}
plot_examples_3d(list(e = e064, t = "e064: lateral_disp = 0"),
                 list(e = e065, t = "e065: lateral_disp = 1"),
                 list(e = e066, t = "e066: lateral_disp = 3"))
```

### Uniform projection placement: `proj_dist_fn = "unif"`

```{r}
e067 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = "unif")
e068 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = "unif")
e069 <- clugen(3, 4, 300, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = "unif")
```

```{r}
plot_examples_3d(list(e = e067, t = "e067: lateral_disp = 0"),
                 list(e = e068, t = "e068: lateral_disp = 1"),
                 list(e = e069, t = "e069: lateral_disp = 3"))
```

### Custom projection placement using the Beta distribution

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

```{r}
e070 <- clugen(3, 4, 400, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = proj_beta)
e071 <- clugen(3, 4, 400, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = proj_beta)
e072 <- clugen(3, 4, 400, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e070, t = "e070: lateral_disp = 0"),
                 list(e = e071, t = "e071: lateral_disp = 1"),
                 list(e = e072, t = "e072: lateral_disp = 3"))
```

## Controlling final point positions from their projections on the cluster-supporting line

```{r}
seed <- 12321
```

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

### Points on hyperplane orthogonal to cluster-supporting line (default): `point_dist_fn = "n-1"`

```{r}
e073 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed)
e074 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               proj_dist_fn = "unif")
e075 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e073, t = "e073: proj_dist_fn = 'norm' (default)"),
                 list(e = e074, t = "e074: proj_dist_fn = 'unif'"),
                 list(e = e075, t = "e075: custom proj_dist_fn (Beta dist.)"))
```

### Points around projection on cluster-supporting line: `point_dist_fn = "n"`

```{r}
e076 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n")
e077 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n", proj_dist_fn = "unif")
e078 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n", proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e076, t = "e076: proj_dist_fn = 'norm' (default)"),
                 list(e = e077, t = "e077: proj_dist_fn = 'unif'"),
                 list(e = e078, t = "e078: custom proj_dist_fn (Beta dist.)"))
```

### Custom point placement using the exponential distribution

```{r}
# Custom point_dist_fn: final points placed using the Exponential distribution
clupoints_n_1_exp <- function(projs, lat_std, len, clu_dir, clu_ctr) {
  dist_exp <- function(npts, lstd) lstd * rexp(npts, rate = 2 / lstd)
  clupoints_n_1_template(projs, lat_std, clu_dir, dist_exp)
}
```

```{r}
e079 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp)
e080 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = "unif")
e081 <- clugen(3, 5, 400, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e079, t = "e079: proj_dist_fn = 'norm' (default)"),
                 list(e = e080, t = "e080: proj_dist_fn = 'unif'"),
                 list(e = e081, t = "e081: custom proj_dist_fn (Beta dist.)"))
```

### Manipulating cluster sizes

```{r}
seed <- 87
```

```{r}
# Custom clucenters_fn (all): yields fixed positions for the clusters
centers_fixed <- function(nclu, csep, coff) {
  matrix(c(-csep[1], -csep[2], -csep[3], csep[1], -csep[2], -csep[3],
           -csep[1], csep[2], csep[3], csep[1], csep[2], csep[3]),
         nrow = nclu, byrow = TRUE)
}

# Custom clusizes_fn (e083): cluster sizes determined via the uniform distribution,
# no correction for total points
clusizes_unif <- function(nclu, npts, ae) sample(2 * npts / nclu, nclu, replace = TRUE)

# Custom clusizes_fn (e084): clusters all have the same size, no correction for
# total points
clusizes_equal <- function(nclu, npts, ae) npts %/% nclu * rep.int(1, nclu)
```

```{r}
e082 <- clugen(3, 4, 400, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               point_dist_fn = "n",
               clucenters_fn = centers_fixed)
e083 <- clugen(3, 4, 400, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               clusizes_fn = clusizes_unif, point_dist_fn = "n",
               clucenters_fn = centers_fixed)
e084 <- clugen(3, 4, 400, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               clusizes_fn = clusizes_equal, point_dist_fn = "n",
               clucenters_fn = centers_fixed)
```

```{r}
plot_examples_3d(list(e = e082, t = "e082: normal dist. (default)"),
                 list(e = e083, t = "e083: unif. dist. (custom)"),
                 list(e = e084, t = "e084: equal size (custom)"))
```

---
title: "Merging and hierarchical cluster examples"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Merging and hierarchical cluster examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{ggplot2,patchwork}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.5,
  dpi = 120
)
```

This section contains several examples on how to merge cluster data, either
generated with **clugenr** or from other sources. Although it is possible to
merge data in any dimension, these examples will focus on merging 2D data.
Therefore, we'll use the same `plot_examples_2d()` function used for the
`vignette("examples2d")`, requiring the following setup code:

```{r setup, message=FALSE, warning=FALSE}
# Load the clugenr library
library(clugenr)

# Load functions for plotting examples
source("plot_examples_2d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

## Merging two data sets generated with `clugen()`

```{r}
seed1 <- 4444
seed2 <- 5555
```

```{r}
e090 <- clugen(2, 5, 1000, c(1, 1), pi / 12, c(20, 20), 14, 1.2, 1.5, seed = seed1,
               proj_dist_fn = "unif", point_dist_fn = "n")
e091 <- clugen(2, 3, 1500, c(1, 0), 0.05, c(20, 20), 0, 0, 4, seed = seed2,
               point_dist_fn = "n", cluster_offset = c(20, 0))
e092 <- clumerge(e090, e091)
```

```{r}
plot_examples_2d(list(e = e090, t = "e090: data set 1"),
                 list(e = e091, t = "e091: data set 2"),
                 list(e = e092, t = "e092: merged data sets"))
```

In the previous example, clusters from individual data sets remain as separate
clusters in the merged data set. It's also possible to maintain the original
cluster labels by setting the `clusters_field` parameter to `NA`:

```{r}
e093 <- clumerge(e090, e091, clusters_field = NA)
```

```{r}
plot_examples_2d(list(e = e090, t = "e090: data set 1"),
                 list(e = e091, t = "e091: data set 2"),
                 list(e = e093, t = "e093: merged data sets"),
                 palette = "set2")
```

## Adding noise to a `clugen()`-generated data set

```{r}
set.seed(333)

e094 <- list(points = matrix(120 * runif(2 * 500) - 60, ncol = 2),
             clusters = factor(rep(9L, 500)))     # Assign noise to cluster 9
e095 <- clumerge(e094, e092, clusters_field = NA) # and keep it 9 when merging
```

```{r}
plot_examples_2d(list(e = e092, t = "e092: original merged data sets"),
                 list(e = e094, t = "e094: random uniform noise"),
                 list(e = e095, t = "e095: data sets with noise"),
                 palette = "set2", pmargin = -0.1)
```

## Merging with data not generated with `clugen()`

Data generated with `clugen()` can be merged with other data sets, for example as
a way of augmenting them. In this example we perform `stats::prcomp()` to
the `datasets::swiss` dataset to reduce its 6 variables to 2. As there are only
47 observations in this dataset, we'll use `clugen()` to generate very similar
clusters and then use `clumerge()` to create an augmented dataset:

```{r}
set.seed(888)
spc <- prcomp(swiss)
e096 <- list(points = spc$x[, 1:2], clusters = factor(rep(4L, dim(spc$x)[1])))
e097 <- clugen(2, 3, 50, c(-0.3, 1), 0., c(0, 0), 0, 0, 3,
               proj_dist_fn = "unif", clusizes_fn = c(20, 6, 25),
               clucenters_fn = matrix(c(-58, -5, 10, -40, 38, 0),
                                      byrow = TRUE, ncol = 2),
               angle_deltas_fn = c(0.02, -0.03, 0.1),
               llengths_fn = c(55, 100, 60))
e098 <- clumerge(e096, e097, clusters_field = NA)
```

```{r}
plot_examples_2d(list(e = e096, t = "e096: swiss PCA"),
                 list(e = e097, t = "e097: clugen-generated"),
                 list(e = e098, t = "e098: merged"),
                 palette = "seaborn")
```

We can also hierarchize clusters from different sources:

```{r}
e099 <- append(e096, list(hclusters = factor(rep(1L, dim(spc$x)[1]))))
e100 <- append(e097, list(hclusters = factor(rep(2L, length(e097$clusters)))))
e101 <- clumerge(e099, e100, fields = "points", clusters_field = "hclusters")
```

```{r}
plot_examples_2d(list(e = e099, t = "e099: swiss PCA (1 cluster)"),
                 list(e = e100, t = "e100: clugen-generated (1 cluster)"),
                 list(e = e101, t = "e101: merged (2 clusters)"),
                 clusters_field = "hclusters",
                 palette = "seaborn")
```

Note that augmenting a dataset this way is probably not a statistically sound
approach for most problems. In any case, if we perform clustering on the merged
dataset and assume there are 3 clusters, results are interesting:

```{r}
# For display purposes, assign all points in merged data to a single cluster
e102 <- list(points = e101$points, hclusters = factor(rep(1L, length(e101$hclusters))))

# Perform hierachical clustering with Manhattan distance, complete linkage and
# assuming 3 clusters
clusts <- cutree(hclust(dist(e101$points, method = "manhattan")), 3)

# Create data set with merged points and new cluster labels
e103 <- list(points = e101$points, hclusters = factor(clusts))
```

```{r}
plot_examples_2d(list(e = e101, t = "e101: merged (2 clusters)"),
                 list(e = e102, t = "e102: merged (1 cluster)"),
                 list(e = e103, t = "e103: after clustering (3 clusters)"),
                 clusters_field = "hclusters",
                 palette = "seaborn")
```

## R CMD check results

0 errors | 0 warnings | 0 notes

## Test environments

* Ubuntu 22.04.2 (R: devel, release, oldrel-1, 3.6.0)
* Windows 11 (R: release)
* macOS 12 (R: release)

## Downstream dependencies

* There are currently no downstream dependencies for this package.

## Other

* Allow direct specification of cluster sizes, cluster centers, line lengths,
  and angle deltas, instead of going through a function.

---
title: "Development"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Development}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  dpi = 120
)
```

```{r setup, include=FALSE}
# Keep examples reproducible in newer R versions and set seed
RNGversion("3.6.0")
set.seed(12345)
```

## Setup `clugenr` development

Clone the package, `cd` into to the package's folder and start R:

```plaintext
$ git clone https://github.com/clugen/clugenr.git
$ cd clugenr
```

Developing `clugenr` requires [devtools], thus make sure it's installed before
continuing. Open an R prompt at the `clugenr` folder and install `clugenr`'s
development dependencies:

```R
devtools::install_dev_deps()
```

The package can be tested as follows:

```R
devtools::test()
```

There are four levels of testing, by increasing thoroughness (and slowness):
`cran`, `ci`, `normal` and `full`. The first two run by default on CRAN and CI
environments, respectively. The third one, `normal`, is the default mode when
running the tests locally, and may take some time (10-20 minutes). The last one,
`full` is more thorough and can take a few hours. It's possible to define the
tests thoroughness by setting the `CLUGENR_TEST_MODE` environment variable. For
example, to simulate how tests would run in CRAN as follows:

```R
Sys.setenv(CLUGENR_TEST_MODE = "cran")
devtools::test()
Sys.unsetenv("CLUGENR_TEST_MODE")
```

The following instruction updates the documentation, then builds and checks the
package locally:

```R
devtools::check()
```

It's also possible to define the environment in which to run the tests when
performing the check:

```R
Sys.setenv(CLUGENR_TEST_MODE = "ci")
devtools::check()
Sys.unsetenv("CLUGENR_TEST_MODE")
```

Create or update the generated documentation in `man/`, file collation and
`NAMESPACE`:

```R
devtools::document()
```

Build `clugenr`'s website (requires [pkgdown], [ggplot2], [patchwork] and
[rgl]):

```R
pkgdown::build_site()
# Can also be done with devtools::build_site()
# In either case add option `preview = F` to avoid opening the browser
```

Install the package locally with:

```R
devtools::install()
```

Check that it works:

```{r, out.width = "100%", dpi = 200}
library(clugenr)
x <- clugen(2, 5, 800, c(-1, 1), 0.6, c(4, 6), 5, 0.4, 0.5)
plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
```

## Code style

To contribute to `clugenr`, follow the [tidyverse style guide]. Some highlights
include, but are not limited to:

* File encoding: UTF-8
* Indentation: 2 spaces (no tabs)
* Line size limit: 80 chars
* Newlines: Unix style, i.e. LF or `\n`

[devtools]: https://devtools.r-lib.org/
[pkgdown]: https://pkgdown.r-lib.org/
[ggplot2]: https://ggplot2.tidyverse.org/
[patchwork]: https://patchwork.data-imaginist.com/index.html
[rgl]: https://dmurdoch.github.io/rgl/
[tidyverse style guide]: https://style.tidyverse.org/index.html

---
title: "Development"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Development}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include=FALSE}
# Keep examples reproducible in newer R versions and set seed
RNGversion("3.6.0")
set.seed(12345)
```

## Setup `clugenr` development

Clone the package, `cd` into to the package's folder and start R:

```plaintext
$ git clone https://github.com/clugen/clugenr.git
$ cd clugenr
```

Developing `clugenr` requires [devtools], thus make sure it's installed before
continuing. Open an R prompt at the `clugenr` folder and install `clugenr`'s
development dependencies:

```R
devtools::install_dev_deps()
```

The package can be tested as follows:

```R
devtools::test()
```

There are four levels of testing, by increasing thoroughness (and slowness):
`cran`, `ci`, `normal` and `full`. The first two run by default on CRAN and CI
environments, respectively. The third one, `normal`, is the default mode when
running the tests locally, and may take some time (10-20 minutes). The last one,
`full` is more thorough and can take a few hours. It's possible to define the
tests thoroughness by setting the `CLUGENR_TEST_MODE` environment variable. For
example, to simulate how tests would run in CRAN as follows:

```R
Sys.setenv(CLUGENR_TEST_MODE = "cran")
devtools::test()
Sys.unsetenv("CLUGENR_TEST_MODE")
```

The following instruction updates the documentation, then builds and checks the
package locally:

```R
devtools::check()
```

It's also possible to define the environment in which to run the tests when
performing the check:

```R
Sys.setenv(CLUGENR_TEST_MODE = "ci")
devtools::check()
Sys.unsetenv("CLUGENR_TEST_MODE")
```

Create or update the generated documentation in `man/`, file collation and
`NAMESPACE`:

```R
devtools::document()
```

Build `clugenr`'s website (requires [pkgdown], [ggplot2], [patchwork] and
[rgl]):

```R
pkgdown::build_site()
# Can also be done with devtools::build_site()
# In either case add option `preview = F` to avoid opening the browser
```

Install the package locally with:

```R
devtools::install()
```

Check that it works:

```{r, out.width = "100%", dpi = 200}
library(clugenr)
x <- clugen(2, 5, 800, c(-1, 1), 0.6, c(4, 6), 5, 0.4, 0.5)
plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
```

## Code style

To contribute to `clugenr`, follow the [tidyverse style guide]. Some highlights
include, but are not limited to:

* File encoding: UTF-8
* Indentation: 2 spaces (no tabs)
* Line size limit: 80 chars
* Newlines: Unix style, i.e. LF or `\n`

[devtools]: https://devtools.r-lib.org/
[pkgdown]: https://pkgdown.r-lib.org/
[ggplot2]: https://ggplot2.tidyverse.org/
[patchwork]: https://patchwork.data-imaginist.com/index.html
[rgl]: https://dmurdoch.github.io/rgl/
[tidyverse style guide]: https://style.tidyverse.org/index.html

---
title: "Theory"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Theory}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette presents a general overview of the _clugen_ algorithm. A complete
description of the algorithm's theoretical framework is available in the article
"[Generating multidimensional clusters with support
lines](https://doi.org/10.1016/j.knosys.2023.110836)" (an open version is
[available on arXiv](https://arxiv.org/abs/2301.10327)).

_Clugen_ is an algorithm for generating multidimensional clusters. Each cluster
is supported by a line segment, the position, orientation and length of which
guide where the respective points are placed. For brevity, *line segments* will
be referred to as *lines*.

Given an $n$-dimensional direction vector $\mathbf{d}$ (and a number of
additional parameters, which will be discussed shortly), the _clugen_ algorithm
works as follows ($^*$ means the algorithm step is stochastic):

1. Normalize $\mathbf{d}$.
2. $^*$Determine cluster sizes.
3. $^*$Determine cluster centers.
4. $^*$Determine lengths of cluster-supporting lines.
5. $^*$Determine angles between $\mathbf{d}$ and cluster-supporting lines.
6. For each cluster:
   1. $^*$Determine direction of the cluster-supporting line.
   2. $^*$Determine distance of point projections from the center of the
      cluster-supporting line.
   3. Determine coordinates of point projections on the cluster-supporting line.
   4. $^*$Determine points from their projections on the cluster-supporting
      line.

Figure 1 provides a stylized overview of the algorithm's steps.

```{asis, echo = crul::ok("https://raw.githubusercontent.com/clugen/.github/main/images/algorithm.png")}
![**Figure 1** - Stylized overview of the *clugen* algorithm. Background tiles
are 10 units wide and tall, when
applicable.](https://raw.githubusercontent.com/clugen/.github/main/images/algorithm.png)
```

The example in Figure 1 was generated with the following parameters:

| Parameter values  | Description               |
|:----------------- | :------------------------ |
| $n=2$           | Number of dimensions.       |
| $c=4$           | Number of clusters.         |
| $p=200$         | Total number of points.     |
| $\mathbf{d}=\begin{bmatrix}1 & 1\end{bmatrix}^T$   | Average direction.          |
| $\theta_\sigma=\pi/16\approx{}11.25^{\circ}$       | Angle dispersion.           |
| $\mathbf{s}=\begin{bmatrix}10 & 10\end{bmatrix}^T$ | Average cluster separation. |
| $l=10$          | Average line length.        |
| $l_\sigma=1.5$  | Line length dispersion.     |
| $f_\sigma=1$    | Cluster lateral dispersion. |

Additionally, all optional parameters (not listed above) were left to their
default values. The complete list of parameters is presented in the `clugen()`
function documentation.

---
title: "Examples in 3D: part I"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Examples in 3D: part I}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{rgl}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.4,
  fig.width = 8.766
)
```

This vignette contains part I of a set of examples on how to use `clugenr` in
3D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
library(clugenr)    # The clugenr library
options(rgl.useNULL = TRUE)  # Create RGL plots in systems without displays (CI)
library(rgl)
setupKnitr(autoprint = TRUE) # Render RGL plots directly on generated page

# Load helper functions for plotting examples
source("plot_examples_3d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 3D examples were plotted with the `plot_examples_3d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_3d.R).

## Manipulating the direction of cluster-supporting lines

### Using the `direction` parameter

```{r}
seed <- 123
```

```{r}
e043 <- clugen(3, 4, 300, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e044 <- clugen(3, 4, 300, c(1, 1, 1), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e045 <- clugen(3, 4, 300, c(0, 0, 1), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
```

```{r}
plot_examples_3d(list(e = e043, t = "e043: direction = [1, 0, 0]"),
                 list(e = e044, t = "e044: direction = [1, 1, 1]"),
                 list(e = e045, t = "e045: direction = [0, 0, 1]"))
```

### Changing the `angle_disp` parameter and using a custom `angle_deltas_fn` function

```{r}
seed <- 123
```

```{r}
# Custom angle_deltas function: arbitrarily rotate some clusters by 90 degrees
angdel_90 <- function(nclu, astd) sample(c(0, pi / 2), nclu, replace = TRUE)
```

```{r}
e046 <- clugen(3, 6, 300, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e047 <- clugen(3, 6, 300, c(1, 0, 0), pi / 8, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e048 <- clugen(3, 6, 300, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed,
               angle_deltas_fn = angdel_90)
```

```{r}
plot_examples_3d(list(e = e046, t = "e046: angle_disp = 0"),
                 list(e = e047, t = "e047: angle_disp = π / 8"),
                 list(e = e048, t = "e048: custom angle_deltas function"))
```

### Specifying a main `direction` for each cluster and changing `angle_disp`

```{r}
seed <- 123
```

```{r}
# Define a main direction for each cluster
dirs <- matrix(c(1, 1, 1,
                 0, 0, 1,
                 1, 0, 0,
                 0, 1, 0,
                 -1, 1, 1),
               nrow = 5, byrow = TRUE)
```

```{r}
e049 <- clugen(3, 5, 400, dirs, 0, rep.int(0, 3), 20, 0, 0.25, seed = seed)
e050 <- clugen(3, 5, 400, dirs, pi / 12, rep.int(0, 3), 20, 0, 0.25, seed = seed)
e051 <- clugen(3, 5, 400, dirs, pi / 4, rep.int(0, 3), 20, 0, 0.25, seed = seed)
```

```{r}
plot_examples_3d(list(e = e049, t = "e049: angle_disp = 0"),
                 list(e = e050, t = "e050: angle_disp = π / 12"),
                 list(e = e051, t = "e051: angle_disp = π / 4"))
```

## Manipulating the length of cluster-supporting lines

```{r}
seed <- 123
```

### Using the `llength` parameter

```{r}
e052 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 0, 0, 0.5,
               seed = seed, point_dist_fn = "n")
e053 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 10, 0, 0.5,
               seed = seed, point_dist_fn = "n")
e054 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 30, 0, 0.5,
               seed = seed, point_dist_fn = "n")
```

```{r}
plot_examples_3d(list(e = e052, t = "e052: llength = 0"),
                 list(e = e053, t = "e053: llength = 10"),
                 list(e = e054, t = "e054: llength = 30"))
```

### Changing the `llength_disp` parameter and using a custom `llengths_fn` function

```{r}
# Custom llengths function: line lengths tend to grow for each new cluster
llen_grow <- function(nclu, llen, llenstd) {
  llen * (0:(nclu - 1) + rnorm(nclu, sd = llenstd))
}
```

```{r}
e055 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 15,  0.0, 0.5,
               seed = seed, point_dist_fn = "n")
e056 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 15, 10.0, 0.5,
               seed = seed, point_dist_fn = "n")
e057 <- clugen(3, 5, 300, c(1, 0, 0), pi / 10, c(10, 10, 10), 10,  0.1, 0.5,
               seed = seed, point_dist_fn = "n", llengths_fn = llen_grow)
```

```{r}
plot_examples_3d(list(e = e055, t = "e055: llength_disp = 0.0"),
                 list(e = e056, t = "e056: llength_disp = 10.0"),
                 list(e = e057, t = "e057: custom llengths function"))
```

## Manipulating relative cluster positions

### Using the `cluster_sep` parameter

```{r}
seed <- 321
```

```{r}
e058 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(30, 10, 10), 25, 4, 3, seed = seed)
e059 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(10, 30, 10), 25, 4, 3, seed = seed)
e060 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(10, 10, 30), 25, 4, 3, seed = seed)
```

```{r}
plot_examples_3d(list(e = e058, t = "e058: cluster_sep = [30, 10, 10]"),
                 list(e = e059, t = "e059: cluster_sep = [10, 30, 10]"),
                 list(e = e060, t = "e060: cluster_sep = [10, 10, 30]"))
```

### Changing the `cluster_offset` parameter and using a custom `clucenters_fn` function

```{r}
seed <- 321
```

```{r}
# Custom clucenters function: places clusters in a diagonal
centers_diag <- function(nclu, csep, coff) {
  matrix(1, nrow = nclu, ncol = length(csep)) * (1:nclu * max(csep)) +
    rep(coff, each = nclu)
}
```

```{r}
e061 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed)
e062 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed,
               cluster_offset = c(20, -20, 20))
e063 <- clugen(3, 8, 300, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed,
               cluster_offset = c(-50, -50, -50), clucenters_fn = centers_diag)
```

```{r}
plot_examples_3d(list(e = e061, t = "e061: default"),
                 list(e = e062, t = "e062: cluster_offset = [20, -20, 20]"),
                 list(e = e063, t = "e063: custom clucenters function"))
```

---
title: "Theory"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Theory}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

This vignette describes the theoretical framework of the _clugen_ algorithm,
starting with a general [Overview](#overview), then moving on to a
[Detailed description](#detailed-description).

## Overview

_Clugen_ is an algorithm for generating multidimensional clusters. Each cluster
is supported by a line segment, the position, orientation and length of which
guide where the respective points are placed. For brevity, *line segments* will
be referred to as *lines*.

Given an $n$-dimensional direction vector $\mathbf{d}$ (and a number of
additional parameters, which will be discussed shortly), the _clugen_ algorithm
works as follows ($^*$ means the algorithm step is stochastic):

1. Normalize $\mathbf{d}$.
2. $^*$Determine cluster sizes.
3. $^*$Determine cluster centers.
4. $^*$Determine lengths of cluster-supporting lines.
5. $^*$Determine angles between $\mathbf{d}$ and cluster-supporting lines.
6. For each cluster:
   1. $^*$Determine direction of the cluster-supporting line.
   2. $^*$Determine distance of point projections from the center of the
      cluster-supporting line.
   3. Determine coordinates of point projections on the cluster-supporting line.
   4. $^*$Determine points from their projections on the cluster-supporting
      line.

Figure 1 provides a stylized overview of the algorithm's steps.

![**Figure 1** - Stylized overview of the *clugen* algorithm. Background tiles
are 10 units wide and tall, when
applicable.](https://raw.githubusercontent.com/clugen/.github/main/images/algorithm.png)

The example in Figure 1 was generated with the following parameters, the exact
meaning of each will be discussed shortly:

| Parameter values  | Description               |
|:----------------- | :------------------------ |
| $n=2$           | Number of dimensions.       |
| $c=4$           | Number of clusters.         |
| $p=200$         | Total number of points.     |
| $\mathbf{d}=\begin{bmatrix}1 & 1\end{bmatrix}^T$   | Average direction.          |
| $\theta_\sigma=\pi/16\approx{}11.25^{\circ}$       | Angle dispersion.           |
| $\mathbf{s}=\begin{bmatrix}10 & 10\end{bmatrix}^T$ | Average cluster separation. |
| $l=10$          | Average line length.        |
| $l_\sigma=1.5$  | Line length dispersion.     |
| $f_\sigma=1$    | Cluster lateral dispersion. |

Additionally, all optional parameters (not listed above) were left to their
default values. These will also be discussed next.

## Detailed description

In this section we provide a detailed description of the algorithm and its
parameters. We start by listing and describing all parameters (mandatory and
optional), and then analyze the algorithm in detail, highlighting how each
parameter influences the end result.

### Algorithm parameters

The _clugen_ algorithm (and consequently, the `clugen()` function) has mandatory
and optional parameters, listed and described in the tables below. The optional
parameters are set to sensible defaults, and in many situations may be left
unchanged. Nonetheless, these allow all of the algorithm's steps to be fully
customized by the user.

#### Mandatory parameters

| Symbol           | Parameter         | Description                                                      |
|:---------------- |:----------------- |:---------------------------------------------------------------- |
| $n$            | `num_dims`        | Number of dimensions.                                            |
| $c$            | `num_clusters`    | Number of clusters.                                              |
| $p$            | `num_points`      | Total number of points to generate.                              |
| $\mathbf{d}$   | `direction`       | Average direction of cluster-supporting lines ($n \times 1$).  |
| $\theta_\sigma$| `angle_disp`      | Angle dispersion of cluster-supporting lines (radians).          |
| $\mathbf{s}$   | `cluster_sep`     | Average cluster separation in each dimension ($n \times 1$).   |
| $l$            | `llength`         | Average length of cluster-supporting lines.                      |
| $l_\sigma$     | `llength_disp`    | Length dispersion of cluster-supporting lines.                   |
| $f_\sigma$     | `lateral_disp`    | Cluster lateral dispersion, i.e., dispersion of points from their projection on the cluster-supporting line. |

#### Optional parameters

| Symbol              | Parameter         | Default value   | Description                                                       |
|:------------------- |:----------------- | :-------------- | :---------------------------------------------------------------- |
| $\phi$            | `allow_empty`     | `FALSE`           | Allow empty clusters?                                             |
| $\mathbf{o}$      | `cluster_offset`  | `vector(mode = "integer", length = num_dims)` | Offset to add to all cluster centers ($n \times 1$). |
| $p_\text{proj}()$ | `proj_dist_fn`    | `"norm"`          | Distribution of point projections along cluster-supporting lines. |
| $p_\text{final}()$| `point_dist_fn`   | `"n-1"`           | Distribution of final points from their projections.              |
| $c_s()$           | `clusizes_fn`     | `clusizes()`      | Distribution of cluster sizes.                                    |
| $c_c()$           | `clucenters_fn`   | `clucenters()`    | Distribution of cluster centers.                                  |
| $l()$             | `llengths_fn`     | `llengths()`      | Distribution of line lengths.                                     |
| $\theta_\Delta()$ | `angle_deltas_fn` | `angle_deltas()`  | Distribution of line angle deltas (w.r.t. $\mathbf{d}$).          |

### The algorithm in detail

The _clugen_ algorithm is presented in [Overview](#overview). In this section we
will analyze each of the algorithms steps in detail.

#### 1. Normalize $\mathbf{d}$

This is a basic step, which consists of converting $\mathbf{d}$ to a unit
vector:

$$
\hat{\mathbf{d}} = \cfrac{\mathbf{d}}{\left\lVert\mathbf{d}\right\rVert}
$$

#### 2. Determine cluster sizes

Cluster sizes are given by the $c_s()$ function according to:

$$
\mathbf{c_s} = c_s(c, p, \phi)
$$

where $\mathbf{c_s}$ is an $c \times 1$ integer vector containing the final
cluster sizes, $c$ is the number of clusters, $p$ is the total number of
points, and $\phi$ is a boolean which determines whether empty clusters are
acceptable.

The $c_s()$ function is an optional parameter, allowing users to customize its
behavior. By default, $c_s()$ is implemented by the `clusizes()` function,
which behaves according to the following algorithm:

1. Determine the size $p_i$ of each cluster $i$ according to
   $p_i\sim\left\lfloor\max\left(\mathcal{N}(\frac{p}{c}, (\frac{p}{3c})^2),0\right)\right\rceil$,
   where $\lfloor\rceil$ denotes the round to nearest integer function, and
    $\mathcal{N}(\mu,\sigma^2)$ represents the normal distribution with
   mean $\mu$ and variance $\sigma^2$.
2. Assure that the final cluster sizes add up to $p$ by incrementing the smallest
   cluster size while $\sum_{i=1}^c p_i<p$ or decrementing the largest cluster
   size while $\sum_{i=1}^c p_i>p$. This step is delegated to the
   `fix_num_points()` helper function.
3. If $\neg\phi\wedge p\ge c$ then, for each empty cluster $i$ (i.e.,
   $p_i=0$), increment $p_i$ and decrement $p_j$, where $j$ denotes the
   largest cluster. This step is delegated to the `fix_empty()` helper function.

Figure 2 demonstrates possible cluster sizes with various definitions of $c_s()$
for $c=4$ and $p=5000$. The default behavior, implemented in the `clusizes()`
function, is shown in Figure 2a, while Figures 2b-d present results obtained
with custom user functions. Figure 2b displays cluster sizes obtained with the
discrete uniform distribution over $\left\{1, 2, \ldots, \frac{2p}{c}\right\}$,
corrected with `fix_num_points()`. In turn, Figure 2c highlights cluster sizes
obtained with the Poisson distribution with $\lambda=\frac{p}{c}$, also corrected
with `fix_num_points()`. The cluster sizes shown in Figure 2d were determined
with the same distribution (Poisson, $\lambda=\frac{p}{c}$), but were not
corrected. Thus, cluster sizes do not add up to $p$, highlighting the fact that
this is not a requirement of the *clugen* algorithm, i.e., user-defined  $c_s()$
implementations can consider $p$ a hint rather than an obligation.

![**Figure 2** - Possible cluster sizes with various definitions of $c_s()$ for
$c=4$ and $p=5000$.](https://raw.githubusercontent.com/clugen/.github/main/images/clusizes.png)

#### 3. Determine cluster centers

Cluster sizes are given by the $c_c()$ function according to:

$$
\mathbf{C} = c_c(c, \mathbf{s}, \mathbf{o})
$$

where $\mathbf{C}$ is an $c \times n$ matrix containing the final cluster
centers, $c$ is the number of clusters, $\mathbf{s}$ is the average cluster
separation ($n \times 1$ vector), and $\mathbf{o}$ is an $n \times 1$ vector
of cluster offsets.

The $c_c()$ function is an optional parameter, allowing users to customize its
behavior. By default, $c_c()$ is implemented by the `clucenters()` function,
which determines the cluster centers according to:

$$
\mathbf{C}=c\mathbf{U} \cdot \operatorname{diag}(\mathbf{s}) + \mathbf{1}\,\mathbf{o}^T
$$

where $\mathbf{U}$ is an $c \times n$ matrix of random values drawn from the
uniform distribution between -0.5 and 0.5, and $\mathbf{1}$ is an $c \times 1$
vector with all entries equal to 1.

Figure 3 shows scatters plots of the results generated by *clugen* for two
different implementations of the $c_c()$ function, namely using the uniform the
distribution (the default, implemented by the `clucenters()` function, Figure
3a), and direct specification of cluster centers (Figure 3b).

![**Figure 3** - The output of *clugen* for two different implementations of the
$c_c()$ function for finding cluster centers: a) the default, using the uniform
distribution; b) hand-picked centers. All parameters are the same as in Figure 1,
except for $p$, which is set to 5000.](https://raw.githubusercontent.com/clugen/.github/main/images/clucenters.png)

#### 4. Determine lengths of cluster-supporting lines

The lengths of the cluster-supporting lines are given by the $l()$ function
according to:

$$
\pmb{\ell} = l(c, l, l_\sigma)
$$

where $\pmb{\ell}$ is an $c \times 1$ vector containing the final lengths of
the cluster-supporting lines, $c$ is the number of clusters, $l$ is the average
length, and $l_\sigma$ is the length dispersion.

The $l()$ function is an optional parameter, allowing users to customize its
behavior. By default, $l()$ is implemented by the `llengths()` function, which
determines the $\ell_i$ length of each cluster-supporting line $i$ according to:

$$
\ell_i\sim\left|\mathcal{N}(l,l_\sigma^2)\right|
$$

where $\left|\mathcal{N}(\mu,\sigma^2)\right|$ represents the folded normal
distribution with mean $\mu$ and variance $\sigma^2$.

Figure 4 shows cluster-supporting line lengths obtained with different implementations
of $l()$.

![**Figure 4** - Line lengths for different implementations of $l()$: a) the
default, using the folded normal distribution; b) using the Poisson distribution,
with $\lambda=l$; c) using the uniform distribution in the interval
$\left\lbrack 0, 2l\right\rbrack$; and, d) hand-picked lengths, more specifically
$\pmb{\ell}=\begin{bmatrix}2 & 8 & 16 & 32\end{bmatrix}^T$. Cluster centers,
as well as parameters $l$ and $l_\sigma$, are the same as for the example
shown in Figure 1.](https://raw.githubusercontent.com/clugen/.github/main/images/llengths.png)

#### 5. Determine angles between $\mathbf{d}$ and cluster-supporting lines

The angles between $\mathbf{d}$ and the cluster-supporting lines are given by
the $\theta_\Delta()$ function according to:

$$
\mathbf{\Theta_\Delta} = \theta_\Delta(c, \theta_\sigma)
$$

where $\mathbf{\Theta_\Delta}$ is an $c \times 1$ vector containing the final
angle differences between $\mathbf{d}$ and the cluster-supporting lines, $c$
is the number of clusters, and $\theta_\sigma$ is the angle dispersion.

The $\theta_\Delta()$ function is an optional parameter, allowing users to
customize its behavior. By default, $\theta_\Delta()$ is implemented by the
`angle_deltas()` function, which determines the $\theta_{\Delta i}$ angle
difference between $\mathbf{d}$ and the $i$-th  cluster-supporting line
according to:

$$
\theta_{\Delta i}\sim\mathcal{WN}_{-\pi/2}^{\pi/2}(0,\theta_\sigma^2)
$$

where $\mathcal{WN}_{-\pi/2}^{\pi/2}(\mu,\sigma^2)$ represents the wrapped normal
distribution with mean $\mu$, variance $\sigma^2$, and support in the
$\left[-\pi/2,\pi/2\right]$ interval, and $\theta_\sigma$ is the angle
dispersion of the cluster-supporting lines.

Figure 5 shows the final direction of the cluster-supporting lines for two
different implementations of $\theta_\Delta()$.

![**Figure 5** - Final directions of the cluster supporting-lines for different
implementations of $\theta_\Delta()$: a) the default, where angle differences
were obtained using the wrapped normal distribution; and, d) hand-picked angle
differences, more specifically
$\mathbf{\Theta_\Delta}=\begin{bmatrix}0 & \frac{\pi}{2} & 0 & \frac{\pi}{2}\end{bmatrix}^T$.
Cluster centers, as well as the angle dispersion $\theta_\sigma$, are the same
as for the example shown in Figure 1.](https://raw.githubusercontent.com/clugen/.github/main/images/angle_deltas.png)

#### 6. For each cluster $i$:

##### 6.1. Determine direction of the cluster-supporting line

In order to obtain the $\hat{\mathbf{d}}_i$ final direction of cluster $i$
supporting line, the following algorithm is used:

* **1.** Find random vector $\mathbf{r}$ with each component taken from the
  uniform distribution between -0.5 and 0.5.
* **2.** Normalize $\mathbf{r}$:
  $$
  \hat{\mathbf{r}}=\cfrac{\mathbf{r}}{\left\lVert\mathbf{r}\right\rVert}
  $$
* **3.** If $|\theta_{\Delta i}| > \pi/2$ or $n=1$, set
  $\hat{\mathbf{d}}_i=\hat{\mathbf{r}}$ and terminate the algorithm.
* **4.** If $\hat{\mathbf{r}}$ is parallel to $\hat{\mathbf{d}}$ go to **1**.
* **5.** Determine vector $\mathbf{d}_\perp$ orthogonal to $\hat{\mathbf{d}}$
  using the first iteration of the Gram-Schmidt process:
  $$
  \mathbf{d}_\perp=\hat{\mathbf{r}}-\cfrac{\hat{\mathbf{d}}\cdot\hat{\mathbf{r}}}{\hat{\mathbf{d}}\cdot\hat{\mathbf{d}}}\:\hat{\mathbf{d}}
  $$
* **6.** Normalize $\mathbf{d}_\perp$:
  $$
  \hat{\mathbf{d}}_\perp=\cfrac{\mathbf{d}_\perp}{\left\lVert\mathbf{d}_\perp\right\rVert}
  $$
* **7.** Determine vector $\mathbf{d}_i$ at angle $\theta_{\Delta i}$ with
  $\hat{\mathbf{d}}$:
  $$
  \mathbf{d}_i=\hat{\mathbf{d}}+\tan(\theta_{\Delta i})\hat{\mathbf{d}}_\perp
  $$
* **8.** Normalize $\mathbf{d}_i$:
  $$
  \hat{\mathbf{d}}_i=\cfrac{\mathbf{d}_i}{\left\lVert\mathbf{d}_i\right\rVert}
  $$

##### 6.2. Determine distance of point projections from the center of the cluster-supporting line

The distance of point projections from the center of the cluster-supporting line
is given by the $p_\text{proj}()$ function according to:

$$
\mathbf{w}_i = p_\text{proj}(\ell_i, p_i)
$$

where $\mathbf{w}_i$ is an $p_i \times 1$ vector containing the distance of
each point projection to the center of the line, while $\ell_i$ and $p_i$ are
the line length and number of points in cluster $i$, respectively.

The $p_\text{proj}()$ function is an optional parameter, allowing users to
customize its behavior. `clugenr` provides two concrete implementations out of
the box, specified by passing `"norm"` or `"unif"` to  `clugen()`'s
`proj_dist_fn` parameter. These work as follows:

* `"norm"` (default) - Each element of $\mathbf{w}_i$ is derived from
  $\mathcal{N}(0, (\frac{\ell_i}{6})^2)$, i.e., from the normal distribution,
  centered on the cluster-supporting line center ($\mu=0$) and with a standard
  deviation of $\sigma=\frac{\ell_i}{6}$, such that the length of the line
  segment encompasses $\approx$ 99.73% of the generated projections. Consequently,
  some projections may be placed outside the line's end points.
* `"unif"` - Each element of $\mathbf{w}_i$ is derived from
  $\mathcal{U}(-\frac{\ell_i}{2}, \frac{\ell_i}{2})$, i.e., from the continuous
  uniform distribution in the interval $\left[-\frac{\ell_i}{2}, \frac{\ell_i}{2}\right[$.
  Thus, projections will be uniformly dispersed along the cluster-supporting line.

The impact of various implementations of $p_\text{proj}()$ is demonstrated in
Figure 6. Figures 6a and 6b show the clusters generated with the `"norm"` and
`"unif"` options, respectively, while Figures 6c and 6d highlight custom user
functions implementing the Laplace and Rayleigh distributions, respectively.
All parameters are set as in Figure 1, except for $p_\text{proj}()$ in the case
of Figures 6b-6d, and $p$, which is set to 5000.

![**Figure 6** - Clusters generated for various implementations of $p_\text{proj}()$:
a) the default, where line center distances are drawn for the normal distribution,
specified using the in-built `"norm"` option; b) in which center distances are
derived from the uniform distribution, via the in-built `"unif"` option; c) where
line center distances are obtained from a custom user function implementing the
Laplace distribution; and, d) in which a custom user function returns center
distances drawn from the Rayleigh distribution. All parameters are set as in Figure
1, except for $p_\text{proj}()$ in the case of Figures 6b-6d, and $p$, which
is set to 5000.](https://raw.githubusercontent.com/clugen/.github/main/images/proj_dist_fn.png)

##### 6.3. Determine coordinates of point projections on the cluster-supporting line

This is a deterministic step performed by the `points_on_line()` function using
the vector formulation of the line equation, as follows:

$$
\mathbf{P}_i^\text{proj}=\mathbf{1}\,\mathbf{c}_i^T + \mathbf{w}_i\hat{\mathbf{d}}_i^T
$$

where $\mathbf{P}_i^\text{proj}$ is the $p_i \times n$ matrix of point projection
coordinates on the line, $\mathbf{1}$ is an $p_i \times 1$ vector with all entries
equal to 1, $\mathbf{c}_i$ are the coordinates of the line center ($n \times 1$
vector), $\mathbf{w}_i$ is the distance of each point projection to the center of
the line ($p_i \times 1$ vector obtained in the previous step), and
$\hat{\mathbf{d}}_i$ is the direction of the cluster-supporting line for cluster
$i$.

##### 6.4. Determine points from their projections on the cluster-supporting line

The final cluster points, obtained from their projections on the cluster-supporting
line, are given by the $p_\text{final}()$ function according to:

$$
\mathbf{P}_i^\text{final} = p_\text{final}(\mathbf{P}_i^\text{proj}, f_\sigma, \ell_i, \hat{\mathbf{d}}_i, \mathbf{c}_i)
$$

where $\mathbf{P}_i^\text{final}$ is a $p_i \times n$ matrix containing the
coordinates of the generated points, $\mathbf{P}_i^\text{proj}$ is the
$p_i \times n$ matrix of projection coordinates (determined in the previous step),
and $f_\sigma$ is the lateral dispersion parameter. In turn, $\ell_i$,
$\hat{\mathbf{d}}_i$ and $\mathbf{c}_i$ are the length, direction and center
of the cluster-supporting line.

The $p_\text{final}()$ function is an optional parameter, allowing users to
customize its behavior. `clugenr` provides two concrete implementations out of
the box, specified by passing `"n-1"` or `"n"` to `clugen()`'s `point_dist_fn`
parameter. These work as follows:

* `"n-1"` (default) - Points are placed on a hyperplane orthogonal to the
  cluster-supporting line and intersecting the point's projection. This is done
  by obtaining $p_i$ random unit vectors orthogonal to $\hat{\mathbf{d}}_i$,
  and determining their magnitude using the normal distribution
  ($\mu=0$, $\sigma=f_\sigma$). These vectors are then added to the respective
  projections on the cluster-supporting line, yielding the final cluster points.
  This behavior is implemented in the `clupoints_n_1()` function.
* `"n"` - Points are placed around their respective projections. This is done by
  obtaining $p_i$ random unit vectors, and determining their magnitude using
  the normal distribution ($\mu=0$, $\sigma=f_\sigma$). These vectors are
  then added to the respective projections on the cluster-supporting line,
  yielding the final cluster points. This behavior is implemented in the
  `clupoints_n()` function.

Figure 7 highlights the differences between these two approaches in 2D, where a
hyperplane is simply a line.

![**Figure 7** - Example of how the final cluster points are obtained in 2D when
using the built-in implementations for $p_\text{final}()$.](https://raw.githubusercontent.com/clugen/.github/main/images/point_dist_ex.png)

In general, points can be placed using a `"n-1"` or `"n"` strategy using any
distribution. Figure 8 displays several examples for various implementations of
$p_\text{final}()$, either based on `"n-1"` or `"n"` strategy, using
different distributions. Figures 8a and 8b show the built-in `"n-1"` and `"n"`
strategies making use of the normal distribution. Figures 8c-8f highlight some
possibilities with custom user functions. Figure 8c shows the effect of using
the exponential distribution in a `"n-1"` strategy, while Figure 8d displays
the result of using a bimodal distribution with the same strategy. A more complex
distribution, producing "hollow" clusters with a `"n"` strategy, is employed in
Figures 8e and 8f, with the latter also having the $p_\text{proj}()$ function
set to `"unif"`. The remaining parameters (for all subfigures) are set as in
Figure 1, except for $p$, which is set to 5000.

![**Figure 8** - Examples of various implementations of $p_\text{final}()$.
Figures a and b shown the effect of the built-in implementations, while Figures
c-f display results obtained using custom user functions.](https://raw.githubusercontent.com/clugen/.github/main/images/point_dist_fn.png)

---
title: "Examples in other dimensions"
output:
  prettydoc::html_pretty:
    theme: tactile
    highlight: vignette
vignette: >
  %\VignetteIndexEntry{Examples in other dimensions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{ggplot2,patchwork}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.5,
  dpi = 120
)
```

This vignette contains a number of examples on how to use `clugenr` in 1D and
5D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
# Load the clugenr library
library(clugenr)

# Load functions for plotting examples
source("plot_examples_1d.R", local = knitr::knit_global())
source("plot_examples_nd.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

## Basic 1D example with density plot

The 1D examples were plotted with the `plot_examples_1d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_1d.R).

```{r}
seed <- 222
```

```{r}
# Custom proj_dist_fn: point projections placed using the Weibull distribution
proj_wbull <- function(len, n) rweibull(n, shape = 1.5, scale = len / 2) - len / 2
```

```{r}
e085 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed)
e086 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed, proj_dist_fn = "unif")
e087 <- clugen(1, 3, 2000, 1, 0, 10, 6, 1.5, 0, seed = seed, proj_dist_fn = proj_wbull)
```

```{r}
plot_examples_1d(list(e = e085, t = "e085: proj_dist_fn='norm'"),
                 list(e = e086, t = "e086: proj_dist_fn='unif'"),
                 list(e = e087, t = "e087: custom proj_dist_fn (Weibull)"),
                 pmargin = 0.07,
                 ymax = 0.725)
```

## 5D example with default optional arguments

The 5D examples were plotted with the `plot_examples_nd()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_nd.R).

```{r}
nd <- 5
seed <- 321
```

```{r}
e088 <- clugen(nd, 6, 1500, c(1, 1, 0.5, 0, 0), pi / 16, rep.int(30, nd), 30, 4, 3,
               seed = seed)
```

```{r, fig.asp = 1}
plot_examples_nd(e088, "e088: 5D with optional parameters set to defaults")
```

## 5D example with `proj_dist_fn = "unif"` and `point_dist_fn = "n"`

```{r}
nd <- 5
seed <- 123
```

```{r}
e089 <- clugen(nd, 6, 1500, c(0.1, 0.3, 0.5, 0.3, 0.1), pi / 12, rep.int(30, nd),
               35, 5, 3.5, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n")
```

```{r, fig.asp = 1}
plot_examples_nd(e089, "e089: 5D with proj_dist_fn='unif' and point_dist_fn='n'")
```

---
title: "Examples in 3D: part II"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples in 3D: part II}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{rgl}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.4,
  fig.width = 8
)
```

This vignette contains part II of a set of examples on how to use `clugenr` in
3D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
library(clugenr)    # The clugenr library
options(rgl.useNULL = TRUE)  # Create RGL plots in systems without displays (CI)
library(rgl)
setupKnitr(autoprint = TRUE) # Render RGL plots directly on generated page

# Load helper functions for plotting examples
source("plot_examples_3d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 3D examples were plotted with the `plot_examples_3d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_3d.R).

## Lateral dispersion and placement of point projections on the line

```{r}
seed <- 456
```

### Normal projection placement (default): `proj_dist_fn = "norm"`

```{r}
e064 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed)
e065 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed)
e066 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed)
```

```{r}
plot_examples_3d(list(e = e064, t = "e064: lateral_disp = 0"),
                 list(e = e065, t = "e065: lateral_disp = 1"),
                 list(e = e066, t = "e066: lateral_disp = 3"))
```

### Uniform projection placement: `proj_dist_fn = "unif"`

```{r}
e067 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = "unif")
e068 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = "unif")
e069 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = "unif")
```

```{r}
plot_examples_3d(list(e = e067, t = "e067: lateral_disp = 0"),
                 list(e = e068, t = "e068: lateral_disp = 1"),
                 list(e = e069, t = "e069: lateral_disp = 3"))
```

### Custom projection placement using the Beta distribution

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

```{r}
e070 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = proj_beta)
e071 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = proj_beta)
e072 <- clugen(3, 4, 1000, c(1, 0, 0), pi / 2, c(20, 20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e070, t = "e070: lateral_disp = 0"),
                 list(e = e071, t = "e071: lateral_disp = 1"),
                 list(e = e072, t = "e072: lateral_disp = 3"))
```

## Controlling final point positions from their projections on the cluster-supporting line

```{r}
seed <- 12321
```

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

### Points on hyperplane orthogonal to cluster-supporting line (default): `point_dist_fn = "n-1"`

```{r}
e073 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed)
e074 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               proj_dist_fn = "unif")
e075 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e073, t = "e073: proj_dist_fn = 'norm' (default)"),
                 list(e = e074, t = "e074: proj_dist_fn = 'unif'"),
                 list(e = e075, t = "e075: custom proj_dist_fn (Beta dist.)"))
```

### Points around projection on cluster-supporting line: `point_dist_fn = "n"`

```{r}
e076 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n")
e077 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n", proj_dist_fn = "unif")
e078 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = "n", proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e076, t = "e076: proj_dist_fn = 'norm' (default)"),
                 list(e = e077, t = "e077: proj_dist_fn = 'unif'"),
                 list(e = e078, t = "e078: custom proj_dist_fn (Beta dist.)"))
```

### Custom point placement using the exponential distribution

```{r}
# Custom point_dist_fn: final points placed using the Exponential distribution
clupoints_n_1_exp <- function(projs, lat_std, len, clu_dir, clu_ctr) {
    dist_exp <- function(npts, lstd) lstd * rexp(npts, rate = 2 / lstd)
    clupoints_n_1_template(projs, lat_std, clu_dir, dist_exp)
}
```

```{r}
e079 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp)
e080 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = "unif")
e081 <- clugen(3, 5, 1500, c(1, 0, 0), pi / 3, c(20, 20, 20), 22, 3, 2, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = proj_beta)
```

```{r}
plot_examples_3d(list(e = e079, t = "e079: proj_dist_fn = 'norm' (default)"),
                 list(e = e080, t = "e080: proj_dist_fn = 'unif'"),
                 list(e = e081, t = "e081: custom proj_dist_fn (Beta dist.)"))
```

### Manipulating cluster sizes

```{r}
seed <- 87
```

```{r}
# Custom clucenters_fn (all): yields fixed positions for the clusters
centers_fixed <- function(nclu, csep, coff) {
  matrix(c(-csep[1], -csep[2], -csep[3], csep[1], -csep[2], -csep[3],
           -csep[1], csep[2], csep[3], csep[1], csep[2], csep[3]),
         nrow = nclu, byrow = TRUE)
}

# Custom clusizes_fn (e083): cluster sizes determined via the uniform distribution,
# no correction for total points
clusizes_unif <- function(nclu, npts, ae) sample(2 * npts / nclu, nclu, replace = TRUE)

# Custom clusizes_fn (e084): clusters all have the same size, no correction for
# total points
clusizes_equal <- function(nclu, npts, ae) npts %/% nclu * rep.int(1, nclu)
```

```{r}
e082 <- clugen(3, 4, 1500, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               point_dist_fn = "n",
               clucenters_fn = centers_fixed)
e083 <- clugen(3, 4, 1500, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               clusizes_fn = clusizes_unif, point_dist_fn = "n",
               clucenters_fn = centers_fixed)
e084 <- clugen(3, 4, 1500, c(1, 1, 1), pi, c(20, 20, 20), 0, 0, 5, seed = seed,
               clusizes_fn = clusizes_equal, point_dist_fn = "n",
               clucenters_fn = centers_fixed)
```

```{r}
plot_examples_3d(list(e = e082, t = "e082: normal dist. (default)"),
                 list(e = e083, t = "e083: unif. dist. (custom)"),
                 list(e = e084, t = "e084: equal size (custom)"))
```

---
title: "Examples in 2D"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples in 2D}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{ggplot2,patchwork}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.5,
  dpi = 200
)
```

This vignette contains a number of examples on how to use `clugenr` in 2D.
Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
# Load the clugenr library
library(clugenr)

# Load functions for plotting examples
source("plot_examples_2d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 2D examples were plotted with the `plot_examples_2d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_2d.R).

## Manipulating the direction of cluster-supporting lines

### Using the `direction` parameter

```{r}
seed <- 123
```

```{r}
e001 <- clugen(2, 4, 200, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e002 <- clugen(2, 4, 200, c(1, 1), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e003 <- clugen(2, 4, 200, c(0, 1), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
```

```{r}
plot_examples_2d(list(e = e001, t = "e001: direction = [1, 0]"),
                 list(e = e002, t = "e002: direction = [1, 1]"),
                 list(e = e003, t = "e003: direction = [0, 1]"))
```

### Changing the `angle_disp` parameter and using a custom `angle_deltas_fn` function

```{r}
seed <- 123
```

```{r}
# Custom angle_deltas function: arbitrarily rotate some clusters by 90 degrees
angdel_90_fn <- function(nclu, astd) sample(c(0, pi / 2), nclu, replace = TRUE)
```

```{r}
e004 <- clugen(2, 6, 500, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed)
e005 <- clugen(2, 6, 500, c(1, 0), pi / 8, c(10, 10), 10, 1.5, 0.5, seed = seed)
e006 <- clugen(2, 6, 500, c(1, 0), 0, c(10, 10), 10, 1.5, 0.5, seed = seed,
               angle_deltas_fn = angdel_90_fn)
```

```{r}
plot_examples_2d(list(e = e004, t = "e004: angle_disp = 0"),
                 list(e = e005, t = "e005: angle_disp = π/8"),
                 list(e = e006, t = "e006: custom angle_deltas function"))
```

## Manipulating the length of cluster-supporting lines

### Using the `llength` parameter

```{r}
seed <- 321
```

```{r}
e007 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 0, 0, 0.5, seed = seed,
               point_dist_fn = "n")
e008 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 10, 0, 0.5, seed = seed,
               point_dist_fn = "n")
e009 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 30, 0, 0.5, seed = seed,
               point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e007, t = "e007: llength = 0"),
                 list(e = e008, t = "e008: llength = 10"),
                 list(e = e009, t = "e009: llength = 30"))
```

### Changing the `llength_disp` parameter and using a custom `llengths_fn` function

```{r}
seed <- 765
```

```{r}
# Custom llengths function: line lengths grow for each new cluster
llen_grow_fn <- function(nclu, llen, llenstd) {
  llen * 0:(nclu - 1) + rnorm(nclu, sd = llenstd)
}
```

```{r}
e010 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 15,  0.0, 0.5,
               seed = seed, point_dist_fn = "n")
e011 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 15, 10.0, 0.5,
               seed = seed, point_dist_fn = "n")
e012 <- clugen(2, 5, 800, c(1, 0), pi / 10, c(10, 10), 10,  0.1, 0.5,
               seed = seed, llengths_fn = llen_grow_fn, point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e010, t = "e010: llength_disp = 0.0"),
                 list(e = e011, t = "e011: llength_disp = 5.0"),
                 list(e = e012, t = "e012: custom llengths function"))
```

## Manipulating relative cluster positions

### Using the `cluster_sep` parameter

```{r}
seed <- 222
```

```{r}
e013 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 15, 2, 4, seed = seed)
e014 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(30, 10), 15, 2, 4, seed = seed)
e015 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 30), 15, 2, 4, seed = seed)
```

```{r}
plot_examples_2d(list(e = e013, t = "e013: cluster_sep = [10, 10]"),
                 list(e = e014, t = "e014: cluster_sep = [30, 10]"),
                 list(e = e015, t = "e015: cluster_sep = [10, 30]"))
```

### Changing the `cluster_offset` parameter and using a custom `clucenters_fn` function

```{r}
seed <- 222
```

```{r}
# Custom clucenters function: places clusters in a diagonal
centers_diag <- function(nclu, csep, coff) {
  matrix(1, nrow = nclu, ncol = length(csep)) * (1:nclu * max(csep)) +
    rep(coff, each = nclu)
}
```

```{r}
e016 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed)
e017 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed,
               cluster_offset = c(20, -20))
e018 <- clugen(2, 8, 1000, c(1, 1), pi / 4, c(10, 10), 10, 2, 2.5, seed = seed,
               cluster_offset = c(-50, -50), clucenters_fn = centers_diag)
```

```{r}
plot_examples_2d(list(e = e016, t = "e016: default"),
                 list(e = e017, t = "e017: cluster_offset = [20, -20]"),
                 list(e = e018, t = "e018: custom clucenters function"))
```

## Lateral dispersion and placement of point projections on the line

```{r}
seed <- 234
```

### Normal projection placement (default): `proj_dist_fn = "norm"`

```{r}
e019 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed)
e020 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed)
e021 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed)
```

```{r}
plot_examples_2d(list(e = e019, t = "e019: lateral_disp = 0"),
                 list(e = e020, t = "e020: lateral_disp = 1"),
                 list(e = e021, t = "e021: lateral_disp = 3"))
```

### Uniform projection placement: `proj_dist_fn = "unif"`

```{r}
e022 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = "unif")
e023 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = "unif")
e024 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = "unif")
```

```{r}
plot_examples_2d(list(e = e022, t = "e022: lateral_disp = 0"),
                 list(e = e023, t = "e023: lateral_disp = 1"),
                 list(e = e024, t = "e024: lateral_disp = 3"))
```

### Custom projection placement using the Beta distribution

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

```{r}
e025 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 0.0, seed = seed,
               proj_dist_fn = proj_beta)
e026 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 1.0, seed = seed,
               proj_dist_fn = proj_beta)
e027 <- clugen(2, 4, 1000, c(1, 0), pi / 2, c(20, 20), 13, 2, 3.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e025, t = "e025: lateral_disp = 0"),
                 list(e = e026, t = "e026: lateral_disp = 1"),
                 list(e = e027, t = "e027: lateral_disp = 3"))
```

## Controlling final point positions from their projections on the cluster-supporting line

```{r}
seed <- 12321
```

```{r}
# Custom proj_dist_fn: point projections placed using the Beta distribution
proj_beta <- function(len, n) len * rbeta(n, 0.1, 0.1) - len / 2
```

### Points on hyperplane orthogonal to cluster-supporting line (default): `point_dist_fn = "n-1"`

```{r}
e028 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed)
e029 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               proj_dist_fn = "unif")
e030 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e028, t = "e028: proj_dist_fn = 'norm' (default)"),
                 list(e = e029, t = "e029: proj_dist_fn = 'unif'"),
                 list(e = e030, t = "e030: custom proj_dist_fn (Beta dist.)"))
```

### Points around projection on cluster-supporting line: `point_dist_fn = "n"`

```{r}
e031 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n")
e032 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n", proj_dist_fn = "unif")
e033 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = "n", proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e031, t = "e031: proj_dist_fn = 'norm' (default)"),
              list(e = e032, t = "e032: proj_dist_fn = 'unif'"),
              list(e = e033, t = "e033: custom proj_dist_fn (Beta dist.)"))
```

### Custom point placement using the exponential distribution

```{r}
# Custom point_dist_fn: final points placed using the Exponential distribution
clupoints_n_1_exp <- function(projs, lat_std, len, clu_dir, clu_ctr) {
    dist_exp <- function(npts, lstd) lstd * rexp(npts, rate = 2 / lstd)
    clupoints_n_1_template(projs, lat_std, clu_dir, dist_exp)
}
```

```{r}
e034 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp)
e035 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = "unif")
e036 <- clugen(2, 5, 1500, c(1, 0), pi / 4, c(20, 20), 14, 2, 2.0, seed = seed,
               point_dist_fn = clupoints_n_1_exp, proj_dist_fn = proj_beta)
```

```{r}
plot_examples_2d(list(e = e034, t = "e034: proj_dist_fn = 'norm' (default)"),
                 list(e = e035, t = "e035: proj_dist_fn = 'unif'"),
                 list(e = e036, t = "e036: custom proj_dist_fn (Beta dist.)"))
```

## Manipulating cluster sizes

```{r}
seed <- 87
```

```{r}
# Custom clucenters_fn (all): yields fixed positions for the clusters
centers_fixed <- function(nclu, csep, coff) {
  matrix(c(-csep[1], -csep[2], csep[1], -csep[2], -csep[1], csep[2], csep[1], csep[2]),
         nrow = nclu, byrow = TRUE)
}

# Custom clusizes_fn (e038): cluster sizes determined via the uniform distribution,
# no correction for total points
clusizes_unif <- function(nclu, npts, ae) sample(2 * npts / nclu, nclu, replace = TRUE)

# Custom clusizes_fn (e039): clusters all have the same size, no correction for
# total points
clusizes_equal <- function(nclu, npts, ae) npts %/% nclu * rep.int(1, nclu)
```

```{r}
e037 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, point_dist_fn = "n")
e038 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, clusizes_fn = clusizes_unif,
               point_dist_fn = "n")
e039 <- clugen(2, 4, 1500, c(1, 1), pi, c(20, 20), 0, 0, 5, seed = seed,
               clucenters_fn = centers_fixed, clusizes_fn = clusizes_equal,
               point_dist_fn = "n")
```

```{r}
plot_examples_2d(list(e = e037, t = "e037: normal dist. (default)"),
                 list(e = e038, t = "e038: unif. dist. (custom)"),
                 list(e = e039, t = "e039: equal size (custom)"))
```

## Direct specification of optional parameters

```{r}
seed <- 9876
```

```{r}
e040 <- clugen(2, 4, 1000, c(-1, 1), 0, c(0, 0), 0, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(50, 200, 500, 2000), llengths_fn = c(0, 2, 4, 6),
               clucenters_fn = matrix(c(-5, -5, -2.5, -2.5, 0, 0, 2.5, 2.5),
                                      nrow = 4, byrow = TRUE))
e041 <- clugen(2, 5, 1000, matrix(c(1, 1, 1, 0, 1, 0, 0, 1, 0, 1),
                                  nrow = 5, byrow = TRUE),
               0, c(0, 0), 0, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(200, 500, 500, 500, 500),
               llengths_fn = c(0, 5, 5, 5, 5),
               clucenters_fn = matrix(c(0, 0, 0, 5, 0, -5, 5, 0, -5, 0),
                                      nrow = 5, byrow = TRUE))
e042 <- clugen(2, 5, 1000, matrix(c(0, 1, 0.25, 0.75, 0.5, 0.5, 0.75, 0.25, 1, 0),
                                  nrow = 5, byrow = TRUE),
               0, c(0, 0), 5, 0, 0.2, seed = seed,
               proj_dist_fn = "unif", point_dist_fn = "n",
               clusizes_fn = c(500, 500, 500, 500, 500),
               clucenters_fn = matrix(c(-5, 0, -3, -0.3, -1, -0.8, 1, -1.6, 3, -2.5),
                                      nrow = 5, byrow = TRUE))
```

```{r}
plot_examples_2d(list(e = e040, t = "e040: direct params 1"),
                 list(e = e041, t = "e041: direct params 2"),
                 list(e = e042, t = "e042: direct params 3"))
```

# clugenr <img src="man/figures/logo.png" align="right" />

**clugenr** is an R package for generating multidimensional clusters. Each
cluster is supported by a line segment, the position, orientation and length of
which guide where the respective points are placed. The `clugen()` function is
provided for this purpose, as well as a number of auxiliary functions, used
internally and modularly by `clugen()`. Users can swap these auxiliary functions
by their own customized versions, fine-tuning their cluster generation
strategies, or even use them as the basis for their own generation algorithms.

## How to install

A stable version of the package is available on CRAN and can be installed with
the following instruction:

```R
install.packages("clugenr")
```

Alternatively, install the development version from GitHub with the following
command (requires the [devtools] package):

```R
devtools::install_github("clugen/clugenr")
```

## Quick start

```R
library(clugenr)
x <- clugen(2, 5, 1000, c(1, -0.5), 0.5, c(4, 6), 6, 0.2, 0.5)
plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
```

![](man/figures/example2d.png)

```R
library(clugenr)
library(rgl) # For 3D plots
x <- clugen(3, 5, 2000, c(1, 0.5, -0.7), 0.5, c(15, 10, 20), 15, 3, 10)
plot3d(x$points, col = x$clusters, xlab = "x", ylab = "y", zlab = "z", aspect = T)
```

![](man/figures/example3d.png)

## Further reading

* [Theory: the _clugen_ algorithm in detail](articles/theory.html)
* Detailed usage examples: [2D](articles/examples2d.html),
  3D ([part I](articles/examples3d1.html), [part II](articles/examples3d2.html)),
  [other dimensions](articles/examplesnd.html)
* [Function reference](reference/index.html)
* [Developing this package](articles/dev.html)

[devtools]: https://cran.r-project.org/package=devtools

---
title: "Examples in 3D: part I"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Examples in 3D: part I}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{rgl}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%",
  fig.asp = 0.4,
  fig.width = 8
)
```

This vignette contains part I of a set of examples on how to use `clugenr` in
3D. Examples require the following setup code:

```{r setup, message=FALSE, warning=FALSE}
library(clugenr)    # The clugenr library
options(rgl.useNULL = TRUE)  # Create RGL plots in systems without displays (CI)
library(rgl)
setupKnitr(autoprint = TRUE) # Render RGL plots directly on generated page

# Load helper functions for plotting examples
source("plot_examples_3d.R", local = knitr::knit_global())

# Keep examples reproducible in newer R versions
RNGversion("3.6.0")
```

The 3D examples were plotted with the `plot_examples_3d()` function available
[here](https://github.com/clugen/clugenr/blob/main/vignettes/plot_examples_3d.R).

## Manipulating the direction of cluster-supporting lines

### Using the `direction` parameter

```{r}
seed <- 123
```

```{r}
e043 <- clugen(3, 4, 500, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e044 <- clugen(3, 4, 500, c(1, 1, 1), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e045 <- clugen(3, 4, 500, c(0, 0, 1), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
```

```{r}
plot_examples_3d(list(e = e043, t = "e043: direction = [1, 0, 0]"),
                 list(e = e044, t = "e044: direction = [1, 1, 1]"),
                 list(e = e045, t = "e045: direction = [0, 0, 1]"))
```

### Changing the `angle_disp` parameter and using a custom `angle_deltas_fn` function

```{r}
seed <- 123
```

```{r}
# Custom angle_deltas function: arbitrarily rotate some clusters by 90 degrees
angdel_90 <- function(nclu, astd) sample(c(0, pi / 2), nclu, replace = TRUE)
```

```{r}
e046 <- clugen(3, 6, 1000, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e047 <- clugen(3, 6, 1000, c(1, 0, 0), pi / 8, c(10, 10, 10), 15, 1.5, 0.5, seed = seed)
e048 <- clugen(3, 6, 1000, c(1, 0, 0), 0, c(10, 10, 10), 15, 1.5, 0.5, seed = seed,
               angle_deltas_fn = angdel_90)
```

```{r}
plot_examples_3d(list(e = e046, t = "e046: angle_disp = 0"),
                 list(e = e047, t = "e047: angle_disp = π / 8"),
                 list(e = e048, t = "e048: custom angle_deltas function"))
```

### Specifying a main `direction` for each cluster and changing `angle_disp`

```{r}
seed <- 123
```

```{r}
# Define a main direction for each cluster
dirs <- matrix(c(1, 1, 1,
                 0, 0, 1,
                 1, 0, 0,
                 0, 1, 0,
                 -1, 1, 1),
               nrow = 5, byrow = TRUE)
```

```{r}
e049 <- clugen(3, 5, 1000, dirs, 0, rep.int(0, 3), 20, 0, 0.25, seed = seed)
e050 <- clugen(3, 5, 1000, dirs, pi / 12, rep.int(0, 3), 20, 0, 0.25, seed = seed)
e051 <- clugen(3, 5, 1000, dirs, pi / 4, rep.int(0, 3), 20, 0, 0.25, seed = seed)
```

```{r}
plot_examples_3d(list(e = e049, t = "e049: angle_disp = 0"),
                 list(e = e050, t = "e050: angle_disp = π / 12"),
                 list(e = e051, t = "e051: angle_disp = π / 4"))
```

## Manipulating the length of cluster-supporting lines

```{r}
seed <- 123
```

### Using the `llength` parameter

```{r}
e052 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 0, 0, 0.5,
               seed = seed, point_dist_fn = "n")
e053 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 10, 0, 0.5,
               seed = seed, point_dist_fn = "n")
e054 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 30, 0, 0.5,
               seed = seed, point_dist_fn = "n")
```

```{r}
plot_examples_3d(list(e = e052, t = "e052: llength = 0"),
                 list(e = e053, t = "e053: llength = 10"),
                 list(e = e054, t = "e054: llength = 30"))
```

### Changing the `llength_disp` parameter and using a custom `llengths_fn` function

```{r}
# Custom llengths function: line lengths tend to grow for each new cluster
llen_grow <- function(nclu, llen, llenstd) {
  llen * (0:(nclu - 1) + rnorm(nclu, sd = llenstd))
}
```

```{r}
e055 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 15,  0.0, 0.5,
               seed = seed, point_dist_fn = "n")
e056 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 15, 10.0, 0.5,
               seed = seed, point_dist_fn = "n")
e057 <- clugen(3, 5, 800, c(1, 0, 0), pi / 10, c(10, 10, 10), 10,  0.1, 0.5,
               seed = seed, point_dist_fn = "n", llengths_fn = llen_grow)
```

```{r}
plot_examples_3d(list(e = e055, t = "e055: llength_disp = 0.0"),
                 list(e = e056, t = "e056: llength_disp = 10.0"),
                 list(e = e057, t = "e057: custom llengths function"))
```

## Manipulating relative cluster positions

### Using the `cluster_sep` parameter

```{r}
seed <- 321
```

```{r}
e058 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(30, 10, 10), 25, 4, 3, seed = seed)
e059 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(10, 30, 10), 25, 4, 3, seed = seed)
e060 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(10, 10, 30), 25, 4, 3, seed = seed)
```

```{r}
plot_examples_3d(list(e = e058, t = "e058: cluster_sep = [30, 10, 10]"),
                 list(e = e059, t = "e059: cluster_sep = [10, 30, 10]"),
                 list(e = e060, t = "e060: cluster_sep = [10, 10, 30]"))
```

### Changing the `cluster_offset` parameter and using a custom `clucenters_fn` function

```{r}
seed <- 321
```

```{r}
# Custom clucenters function: places clusters in a diagonal
centers_diag <- function(nclu, csep, coff) {
  matrix(1, nrow = nclu, ncol = length(csep)) * (1:nclu * max(csep)) +
    rep(coff, each = nclu)
}
```

```{r}
e061 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed)
e062 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed,
               cluster_offset = c(20, -20, 20))
e063 <- clugen(3, 8, 1000, c(1, 1, 1), pi / 4, c(10, 10, 10), 12, 3, 2.5, seed = seed,
               cluster_offset = c(-50, -50, -50), clucenters_fn = centers_diag)
```

```{r}
plot_examples_3d(list(e = e061, t = "e061: default"),
                 list(e = e062, t = "e062: cluster_offset = [20, -20, 20]"),
                 list(e = e063, t = "e063: custom clucenters function"))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{clugen}
\alias{clugen}
\title{Generate multidimensional clusters}
\usage{
clugen(
  num_dims,
  num_clusters,
  num_points,
  direction,
  angle_disp,
  cluster_sep,
  llength,
  llength_disp,
  lateral_disp,
  allow_empty = FALSE,
  cluster_offset = NA,
  proj_dist_fn = "norm",
  point_dist_fn = "n-1",
  clusizes_fn = clusizes,
  clucenters_fn = clucenters,
  llengths_fn = llengths,
  angle_deltas_fn = angle_deltas,
  seed = NA
)
}
\arguments{
\item{num_dims}{Number of dimensions.}

\item{num_clusters}{Number of clusters to generate.}

\item{num_points}{Total number of points to generate.}

\item{direction}{Average direction of the cluster-supporting lines. Can be
a vector of length \code{num_dims} (same direction for all clusters) or a
matrix of size \code{num_clusters} x \code{num_dims} (one direction per cluster).}

\item{angle_disp}{Angle dispersion of cluster-supporting lines (radians).}

\item{cluster_sep}{Average cluster separation in each dimension (vector of
length \code{num_dims}).}

\item{llength}{Average length of cluster-supporting lines.}

\item{llength_disp}{Length dispersion of cluster-supporting lines.}

\item{lateral_disp}{Cluster lateral dispersion, i.e., dispersion of points
from their projection on the cluster-supporting line.}

\item{allow_empty}{Allow empty clusters? \code{FALSE} by default.}

\item{cluster_offset}{Offset to add to all cluster centers (vector of length
\code{num_dims}). By default there will be no offset.}

\item{proj_dist_fn}{Distribution of point projections along
cluster-supporting lines, with three possible values:
\itemize{
\item \code{"norm"} (default): Distribute point projections along lines using a normal
distribution (\mjeqn{\mu=}{μ=} \emph{line_center},
\mjeqn{\sigma=}{σ=} \code{llength/6} ).
\item \code{"unif"}: Distribute points uniformly along the line.
\item User-defined function, which accepts two parameters, line length (\code{double})
and number of points (\code{integer}), and returns a vector containing the
distance of each point projection to the center of the line. For example,
the \code{"norm"} option roughly corresponds to
\code{function(l, n) stats::rnorm(n, sd = l / 6)}.
}}

\item{point_dist_fn}{Controls how the final points are created from their
projections on the cluster-supporting lines, with three possible values:
\itemize{
\item \code{"n-1"} (default): Final points are placed on a hyperplane orthogonal to
the cluster-supporting line, centered at each point's projection, using the
normal distribution (\mjeqn{\mu=0}{μ=0},
\mjeqn{\sigma=}{σ=} \code{lateral_disp} ). This is done by the \link{clupoints_n_1}
function.
\item \code{"n"}: Final points are placed around their projection on the
cluster-supporting line using the normal distribution (\mjeqn{\mu=0}{μ=0},
\mjeqn{\sigma=}{σ=} \code{lateral_disp} ). This is done by the \link{clupoints_n}
function.
\item User-defined function: The user can specify a custom point placement
strategy by passing a function with the same signature as \link{clupoints_n_1}
and \link{clupoints_n}.
}}

\item{clusizes_fn}{Distribution of cluster sizes. By default, cluster sizes
are determined by the \link{clusizes} function, which uses the normal distribution
(\mjeqn{\mu=}{μ=} \code{num_points}/\code{num_clusters}, \mjeqn{\sigma=\mu/3}{σ=μ/3}),
and assures that the final cluster sizes add up to \code{num_points}. This
parameter allows the user to specify a custom function for this purpose,
which must follow \link{clusizes} signature. Note that custom functions are not
required to strictly obey the \code{num_points} parameter. Alternatively, the user
can specify a vector of cluster sizes directly.}

\item{clucenters_fn}{Distribution of cluster centers. By default, cluster
centers are determined by the \link{clucenters} function, which uses the uniform
distribution, and takes into account the \code{num_clusters} and \code{cluster_sep}
parameters for generating well-distributed cluster centers. This parameter
allows the user to specify a custom function for this purpose, which must
follow \link{clucenters} signature. Alternatively, the user can specify a matrix
of size \code{num_clusters} x \code{num_dims} with the exact cluster centers.}

\item{llengths_fn}{Distribution of line lengths. By default, the lengths of
cluster-supporting lines are determined by the \link{llengths} function, which
uses the folded normal distribution (\mjeqn{\mu=}{μ=} \code{llength},
\mjeqn{\sigma=}{σ=} \code{llength_disp} ). This parameter allows the user to
specify a custom function for this purpose, which must follow \link{llengths}
signature. Alternatively, the user can specify a vector of line lengths
directly.}

\item{angle_deltas_fn}{Distribution of line angle differences with respect to
\code{direction}. By default, the angles between the main \code{direction} of each
cluster and the final directions of their cluster-supporting lines are
determined by the \link{angle_deltas} function, which uses the wrapped normal
distribution (\mjeqn{\mu=0}{μ=0}, \mjeqn{\sigma=}{σ=} \code{angle_disp} ) with
support in the interval \mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}. This
parameter allows the user to specify a custom function for this purpose,
which must follow \link{angle_deltas} signature. Alternatively, the user can
specify a vector of angle deltas directly.}

\item{seed}{An integer used to initialize the PRNG, allowing for reproducible
results. If specified, \code{seed} is simply passed to \link{set.seed}.}
}
\value{
A named list with the following elements:
\itemize{
\item \code{points}: A \code{num_points} x \code{num_dims} matrix with the generated points for
all clusters.
\item \code{clusters}: A \code{num_points} factor vector indicating which cluster
each point in \code{points} belongs to.
\item \code{projections}: A \code{num_points} x \code{num_dims} matrix with the point
projections on the cluster-supporting lines.
\item \code{sizes}: A \code{num_clusters} x 1 vector with the number of points in
each cluster.
\item \code{centers}: A \code{num_clusters} x \code{num_dims} matrix with the
coordinates of the cluster centers.
\item \code{directions}: A \code{num_clusters} x \code{num_dims} matrix with the final
direction of each cluster-supporting line.
\item \code{angles}: A \code{num_clusters} x 1 vector with the angles between the
cluster-supporting lines and the main direction.
\item \code{lengths}: A \code{num_clusters} x 1 vector with the lengths of the
cluster-supporting lines.
}
}
\description{
\loadmathjax
This is the main function of \strong{clugenr}, and possibly the only function most
users will need.
}
\details{
If a custom function was given in the \code{clusizes_fn} parameter, it is
possible that \code{num_points} may have a different value than what was
specified in the \code{num_points} parameter.

The terms "average" and "dispersion" refer to measures of central
tendency and statistical dispersion, respectively. Their exact meaning
depends on the optional arguments.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
# 2D example
x <- clugen(2, 5, 1000, c(1, 3), 0.5, c(10, 10), 8, 1.5, 2)
graphics::plot(x$points, col = x$clusters, xlab = "x", ylab = "y", asp = 1)
# 3D example
x <- clugen(3, 5, 1000, c(2, 3, 4), 0.5, c(15, 13, 14), 7, 1, 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{fix_empty}
\alias{fix_empty}
\title{Certify that, given enough points, no clusters are left empty}
\usage{
fix_empty(clu_num_points, allow_empty = FALSE)
}
\arguments{
\item{clu_num_points}{Number of points in each cluster (vector of size
\mjseqn{c}), where \mjseqn{c} is the number of clusters.}

\item{allow_empty}{Allow empty clusters?}
}
\value{
Number of points in each cluster, after being fixed by this function
(vector of size \mjseqn{c}).
}
\description{
\loadmathjax
Certifies that, given enough points, no clusters are left empty. This is done
by removing a point from the largest cluster and adding it to an empty
cluster while there are empty clusters. If the total number of points is
smaller than the number of clusters (or if the \code{allow_empty} parameter is set
to \code{TRUE}), this function does nothing.
}
\details{
This function is used internally by \link{clusizes} and might be useful for custom
cluster sizing implementations given as the \code{clusizes_fn} parameter of the
main \link{clugen} function.
}
\examples{
clusters <- c(3, 4, 5, 0, 0)    # A vector with some empty elements
clusters <- fix_empty(clusters) # Apply this function
clusters                        # Check that there's no more empty elements
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module.R
\name{clusizes}
\alias{clusizes}
\title{Determine cluster sizes, i.e., the number of points in each cluster}
\usage{
clusizes(num_clusters, num_points, allow_empty)
}
\arguments{
\item{num_clusters}{Number of clusters.}

\item{num_points}{Total number of points.}

\item{allow_empty}{Allow empty clusters?}
}
\value{
Number of points in each cluster (vector of length \code{num_clusters}).
}
\description{
\loadmathjax
Cluster sizes are determined using the normal distribution
(\mjeqn{\mu=}{μ=} \code{num_points} \mjseqn{/} \code{num_clusters},
\mjeqn{\sigma=\mu/3}{σ=μ/3}), and then assuring that the final cluster sizes
add up to \code{num_points} via the \link{fix_num_points} function.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(123)
sizes <- clusizes(4, 1000, TRUE)
sizes
sum(sizes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module.R
\name{angle_deltas}
\alias{angle_deltas}
\title{Get angles between average cluster direction and cluster-supporting lines}
\usage{
angle_deltas(num_clusters, angle_disp)
}
\arguments{
\item{num_clusters}{Number of clusters.}

\item{angle_disp}{Angle dispersion, in radians.}
}
\value{
Angles between the average cluster direction and the
cluster-supporting lines, given in radians in the interval
\mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}
}
\description{
\loadmathjax
Determine the angles between the average cluster direction and the
cluster-supporting lines. These angles are obtained from a wrapped normal
distribution (\mjeqn{\mu=0}{μ=0}, \mjeqn{\sigma=}{σ=} \code{angle_disp} ) with
support in the interval \mjeqn{\left[-\pi/2,\pi/2\right]}{[-π/2, π/2]}.
Note this is different from the standard wrapped normal distribution, the
support of which is given by the interval
\mjeqn{\left[-\pi,\pi\right]}{[-π, π]}.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(123)
arad <- angle_deltas(4, pi / 8) # Angle dispersion of 22.5 degrees
arad                            # What angles deltas did we get?
arad * 180 / pi                 #  Show angle deltas in degrees
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{points_on_line}
\alias{points_on_line}
\title{Determine coordinates of points on a line}
\usage{
points_on_line(center, direction, dist_center)
}
\arguments{
\item{center}{Center of the line (\mjseqn{n}-component vector).}

\item{direction}{Line direction (\mjseqn{n}-component unit vector).}

\item{dist_center}{Distance of each point to the center of the line
(\mjseqn{n}-component vector, where \mjseqn{n} is the number of points).}
}
\value{
Coordinates of points on the specified line
(\mjeqn{p \times n}{p x n} matrix).
}
\description{
\loadmathjax
Determine coordinates of points on a line with \code{center} and \code{direction},
based on the distances from the center given in \code{dist_center}.

This works by using the vector formulation of the line equation assuming
\code{direction} is a \mjseqn{n}-dimensional unit vector. In other words,
considering \mjeqn{\mathbf{d}=}{d =} \code{as.matrix(direction)} (\mjeqn{n \times
1}{n x 1} vector), \mjeqn{\mathbf{c}=}{c =} \code{as.matrix(center)} (\mjeqn{n
\times 1}{n x 1} vector), and \mjeqn{\mathbf{w}=}{w =}
\code{as.matrix(dist_center)} (\mjeqn{p \times 1}{p x 1} vector), the coordinates
of points on the line are given by:

\mjdeqn{\mathbf{P}=\mathbf{1}\,\mathbf{c}^T + \mathbf{w}\mathbf{d}^T}{
P = 1c' + wd'}

where \mjeqn{\mathbf{P}}{P} is the \mjeqn{p \times n}{p x n} matrix of point
coordinates on the line, and \mjeqn{\mathbf{1}}{1} is a \mjeqn{p \times 1}{p
x 1} vector with all entries equal to 1.
}
\examples{
points_on_line(c(5, 5), c(1, 0), seq(-4, 4, length.out=5)) # 2D, 5 points

points_on_line(c(-2, 0, 0, 2), c(0, 0, -1, 0), c(10, -10)) # 4D, 2 points
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{rand_vector_at_angle}
\alias{rand_vector_at_angle}
\title{Get a random unit vector at a given angle with another vector.}
\usage{
rand_vector_at_angle(u, angle)
}
\arguments{
\item{u}{Unit vector with \mjseqn{n} components.}

\item{angle}{Angle in radians.}
}
\value{
Random unit vector with \mjseqn{n} components which is at \code{angle}
radians with vector \code{u}.
}
\description{
\loadmathjax
Get a random unit vector which is at \code{angle} radians of vector \code{u}.
Note that \code{u} is expected to be a unit vector itself.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
u <- c(1.0, 0, 0.5, -0.5)            # Define a 4D vector
u <- u / norm(u, "2")                # Normalize the vector
v <- rand_vector_at_angle(u, pi / 4) # Get a vector at 45 degrees
arad <- acos((u \%*\% v) / norm(u,"2") * norm(v, "2")) # Get angle in radians
arad * 180 / pi # Convert to degrees, should be close to 45 degrees
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{clupoints_n_1_template}
\alias{clupoints_n_1_template}
\title{Create points from their projections on a cluster-supporting line}
\usage{
clupoints_n_1_template(projs, lat_disp, clu_dir, dist_fn)
}
\arguments{
\item{projs}{Point projections on the cluster-supporting line
(\mjeqn{p \times n}{p x n} matrix).}

\item{lat_disp}{Dispersion of points from their projection.}

\item{clu_dir}{Direction of the cluster-supporting line (unit vector).}

\item{dist_fn}{Function to place points on a second line, orthogonal to the
first.}
}
\value{
Generated points (\mjeqn{p \times n}{p x n} matrix).
}
\description{
\loadmathjax
Generate points from their \mjseqn{n}-dimensional projections on a
cluster-supporting line, placing each point on a hyperplane orthogonal to
that line and centered at the point's projection. The function specified in
\code{dist_fn} is used to perform the actual placement.
}
\details{
This function is used internally by \link{clupoints_n_1} and may be useful for
constructing user-defined final point placement strategies for the
\code{point_dist_fn} parameter of the main \link{clugen} function.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(123)
ctr <- c(0, 0)
dir <- c(1, 0)
pdist <- c(-0.5, -0.2, 0.1, 0.3)
proj <- points_on_line(ctr, dir, pdist)
clupoints_n_1_template(proj, 0, dir, function(p, l) stats::runif(p))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{fix_num_points}
\alias{fix_num_points}
\title{Certify that array values add up to a specific total}
\usage{
fix_num_points(clu_num_points, num_points)
}
\arguments{
\item{clu_num_points}{Number of points in each cluster (vector of size
\mjseqn{c}), where \mjseqn{c} is the number of clusters.}

\item{num_points}{The expected total number of points.}
}
\value{
Number of points in each cluster, after being fixed by this function.
}
\description{
\loadmathjax
Certifies that the values in the \code{clu_num_points} array, i.e. the number of
points in each cluster, add up to \code{num_points}. If this is not the case, the
\code{clu_num_points} array is modified in-place, incrementing the value
corresponding to the smallest cluster while
\code{sum(clu_num_points) < num_points}, or decrementing the value corresponding
to the largest cluster while \code{sum(clu_num_points) > num_points}.
}
\details{
This function is used internally by \link{clusizes} and might be useful for
custom cluster sizing implementations given as the \code{clusizes_fn} parameter of
the main \link{clugen} function.
}
\examples{
clusters <- c(1, 6, 3)                   # 10 total points
clusters <- fix_num_points(clusters, 12) # But we want 12 total points
clusters                                 # Check that we now have 12 points
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module.R
\name{clucenters}
\alias{clucenters}
\title{Determine cluster centers using the uniform distribution}
\usage{
clucenters(num_clusters, clu_sep, clu_offset)
}
\arguments{
\item{num_clusters}{Number of clusters.}

\item{clu_sep}{Average cluster separation (\mjeqn{n \times 1}{n x 1} vector).}

\item{clu_offset}{Cluster offsets (\mjeqn{n \times 1}{n x 1} vector).}
}
\value{
A \mjeqn{c \times n}{c x n} matrix containing the cluster centers.
}
\description{
\loadmathjax
Determine cluster centers using the uniform distribution, taking into account
the number of clusters (\code{num_clusters}) and the average cluster separation
(\code{clu_sep}).

More specifically, let \mjseqn{c=}\code{num_clusters},
\mjeqn{\mathbf{s}=}{s=}\code{clu_sep}, \mjeqn{\mathbf{o}=}{o=}\code{clu_offset},
\mjseqn{n=}\code{length(clu_sep)} (i.e., number of dimensions). Cluster centers
are obtained according to the following equation:

\mjdeqn{\mathbf{C}=c\mathbf{U}\cdot\operatorname{diag}(\mathbf{s}) +
\mathbf{1}\,\mathbf{o}^T}{C=cU.diag(s) + 1o'}

where \mjeqn{\mathbf{C}}{C} is the \mjeqn{c \times n}{c x n} matrix of
cluster centers, \mjeqn{\mathbf{U}}{U} is an \mjeqn{c \times n}{c x n} matrix
of random values drawn from the uniform distribution between -0.5 and 0.5,
and \mjeqn{\mathbf{1}}{1} is an \mjeqn{c \times 1}{c x 1} vector with all
entries equal to 1.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(321)
clucenters(3, c(30, 10), c(-50,50))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{rand_ortho_vector}
\alias{rand_ortho_vector}
\title{Get a random unit vector orthogonal to \code{u}.}
\usage{
rand_ortho_vector(u)
}
\arguments{
\item{u}{A unit vector.}
}
\value{
A random unit vector orthogonal to \code{u}.
}
\description{
Get a random unit vector orthogonal to \code{u}.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
r <- stats::runif(3)      # Get a random 3D vector
r <- r / norm(r, "2")     # Normalize it
o <- rand_ortho_vector(r) # Get a random unit vector orthogonal to r
r \%*\% o            # Check that r and o are orthogonal (result should be ~0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module.R
\name{clupoints_n_1}
\alias{clupoints_n_1}
\title{Create points from their projections on a cluster-supporting line}
\usage{
clupoints_n_1(projs, lat_disp, line_len, clu_dir, clu_ctr)
}
\arguments{
\item{projs}{Point projections on the cluster-supporting line
(\mjeqn{p \times n}{p x n} matrix).}

\item{lat_disp}{Standard deviation for the normal distribution, i.e., cluster
lateral dispersion.}

\item{line_len}{Length of cluster-supporting line (ignored).}

\item{clu_dir}{Direction of the cluster-supporting line.}

\item{clu_ctr}{Center position of the cluster-supporting line (ignored).}
}
\value{
Generated points (\mjeqn{p \times n}{p x n} matrix).
}
\description{
\loadmathjax
Each point is placed on a hyperplane orthogonal to that line and centered at
the point's projection, using the normal distribution (\mjeqn{\mu=0}{μ=0},
\mjeqn{\sigma=}{σ=} \code{lat_disp} ).
}
\details{
This function's main intended use is by the main \link{clugen} function,
generating the final points when the \code{point_dist_fn} parameter is set to
\code{"n-1"}.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(123)
ctr <- c(0, 0)
dir <- c(1, 0)
pdist <- c(-0.5, -0.2, 0.1, 0.3)
proj <- points_on_line(ctr, dir, pdist)
clupoints_n_1(proj, 0.1, NA, dir, NA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\name{rand_unit_vector}
\alias{rand_unit_vector}
\title{Get a random unit vector with \code{num_dims} components.}
\usage{
rand_unit_vector(num_dims)
}
\arguments{
\item{num_dims}{Number of components in vector (i.e. vector size).}
}
\value{
A random unit vector with \code{num_dims} components.
}
\description{
Get a random unit vector with \code{num_dims} components.
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
r <- rand_unit_vector(4)
norm(r, "2")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clugenr.R
\docType{package}
\name{clugenr}
\alias{clugenr}
\title{clugenr: Multidimensional cluster generation in R}
\description{
clugenr is an R package for generating multidimensional clusters. Each
cluster is supported by a line segment, the position, orientation and length
of which guide where the respective points are placed.
}
\note{
MIT License
}
\author{
Nuno Fachada
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/module.R
\name{llengths}
\alias{llengths}
\title{Determine length of cluster-supporting lines}
\usage{
llengths(num_clusters, llength, llength_disp)
}
\arguments{
\item{num_clusters}{Number of clusters.}

\item{llength}{Average line length.}

\item{llength_disp}{Line length dispersion.}
}
\value{
Lengths of cluster-supporting lines (vector of size \code{num_clusters}).
}
\description{
\loadmathjax
Line lengths are determined using the folded normal distribution
(\mjeqn{\mu=}{μ=} \code{llength}, \mjeqn{\sigma=}{σ=} \code{llength_disp} ).
}
\note{
This function is stochastic. For reproducibility set a PRNG seed with
\link{set.seed}.
}
\examples{
set.seed(123)
llengths(4, 20, 3.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{angle_btw}
\alias{angle_btw}
\title{Angle between two \mjseqn{n}-dimensional vectors.}
\usage{
angle_btw(v1, v2)
}
\arguments{
\item{v1}{First vector.}

\item{v2}{Second vector.}
}
\value{
Angle between \code{v1} and \code{v2} in radians.
}
\description{
\loadmathjax
Typically, the angle between two vectors \code{v1} and \code{v2} can be obtained with:

\if{html}{\out{<div class="sourceCode R">}}\preformatted{acos((v1 \%*\% v2) / (norm(v1, "2") * norm(v2, "2")))
}\if{html}{\out{</div>}}

However, this approach is numerically unstable. The version provided here is
numerically stable and based on the \href{https://github.com/JeffreySarnoff/AngleBetweenVectors.jl}{Angle Between Vectors}
Julia package by Jeffrey Sarnoff (MIT license), implementing an algorithm
provided by Prof. W. Kahan in
\href{https://people.eecs.berkeley.edu/~wkahan/MathH110/Cross.pdf}{these notes}
(see page 15).
}
\examples{
angle_btw(c(1.0, 1.0, 1.0, 1.0), c(1.0, 0.0, 0.0, 0.0)) * 180 / pi
}
